Global _g_ib3d_draw_manager:DRAW_MANAGER

New DRAW_MANAGER

Type DRAW_MANAGER
	Field drawList:TList
	
	Method New()
		_g_ib3d_draw_manager=Self
		drawList=New TList
	EndMethod
	
	Method Draw()
		If drawList.Count()>0
			' setup the material and matrix
			Local material:SMaterial=SMaterial.create()
			material.setLighting(False)
			_g_ib3d_engine.driver.SetTransform(ETS_WORLD,Matrix4.create())
			_g_ib3d_engine.driver.setMaterial(material)
	
			' draw all queued commands
			For Local drawCmd:ib3d_IDrawCmd=EachIn drawList
				drawCmd.Draw()
			Next
			drawList.Clear()		
		EndIf
	EndMethod

	Method QueueDrawTextCmd(text:String,x:Int,y:Int)		
		Local cmd:ib3d_DrawTextCmd=generateDrawTextCmd()
		cmd.cmdText=text
		cmd.cmdX=x
		cmd.cmdY=y		
		cmd.cmdFont=_g_ib3d_font_manager.current
		cmd.cmdColor=_g_ib3d_font_manager._color
		drawList.AddLast(cmd)
	EndMethod

	Method QueueDrawLineCmd(line:Line3df,color:SColor=Null)
		If line And line.IsValid()
			Local cmd:ib3d_DrawLineCmd=generateDrawLineCmd()
			cmd.cmdLine=line
			cmd.cmdColor=color
			drawList.AddLast(cmd)
		EndIf
	EndMethod

	Method QueueDrawTriangleCmd(triangle:Triangle3df,color:SColor=Null)
		If triangle And triangle.IsValid()
			Local cmd:ib3d_DrawTriangleCmd=generateDrawTriangleCmd()
			cmd.cmdTri=triangle
			cmd.cmdColor=color
			drawList.AddLast(cmd)		
		EndIf
	EndMethod
	
	Method generateDrawTextCmd:ib3d_DrawTextCmd()
		Return New ib3d_DrawTextCmd
	EndMethod
	
	Method generateDrawLineCmd:ib3d_DrawLineCmd()
		Return New ib3d_DrawLineCmd
	EndMethod
	
	Method generateDrawTriangleCmd:ib3d_DrawTriangleCmd()
		Return New ib3d_DrawTriangleCmd
	EndMethod
EndType

' interface for drawing commands
Type ib3d_IDrawCmd
	Method Draw() Abstract
EndType

Type ib3d_DrawTextCmd Extends ib3d_IDrawCmd
	Field cmdText:String
	Field cmdX:Int
	Field cmdY:Int
	Field cmdFont:FONT=Null
	Field cmdColor:SColor=Null
	
	Method Draw()
		If _g_ib3d_font_manager.GetCurrentFont()<>cmdFont Then _g_ib3d_font_manager.SetCurrentFont(cmdFont)
		If _g_ib3d_font_manager._color<>cmdColor Then _g_ib3d_font_manager._color=cmdColor		
		_g_ib3d_font_manager.DrawText(cmdText,cmdX,cmdY)
	EndMethod
EndType

Type ib3d_DrawLineCmd Extends ib3d_IDrawCmd
	Field cmdLine:Line3df
	Field cmdColor:SColor
		
	Method Draw()
		_g_ib3d_engine.driver.draw3dLine(cmdLine.getStart(),cmdLine.getEnd(),cmdColor)
	EndMethod
EndType

Type ib3d_DrawTriangleCmd Extends ib3d_IDrawCmd
	Field cmdTri:Triangle3df
	Field cmdColor:SColor

	Method Draw()
		_g_ib3d_engine.driver.draw3dTriangle(cmdTri,cmdColor)
	EndMethod
EndType

Rem
bbdoc: Draws Text on the screen using the current font and color
about: <i>Parameters</i>
<ul>
<li>text:String - the text to draw</li>
<li>x:Int - X location for the upper left of the text</li>
<li>y:Int - Y location for the upper left of the text</li>
</ul>
EndRem
Function ib3d_DrawText(text:String,x:Int,y:Int)
	If Not _g_ib3d_draw_manager Then RuntimeError "iB3D draw manager has not been initialized"
	If Not _g_ib3d_font_manager Then RuntimeError "iB3D font manager has not been initialized"
	_g_ib3d_draw_manager.QueueDrawTextCmd(text,x,y)
EndFunction

Rem
bbdoc: Draws a 3d line on the screen from provided points
about: <i>Parameters</i>
<ul>
<li>x1:Float - X location for the start of the line</li>
<li>y1:Float - Y location for the start of the line</li>
<li>z1:Float - Z location for the start of the line</li>
<li>x2:Float - X location for the end of the line</li>
<li>y2:Float - Y location for the end of the line</li>
<li>z2:Float - Z location for the end of the line</li>
<li>red:Int - red component of the color to draw the line with.  Defaults to 255.</li>
<li>green:Int - green component of the color to draw the line with.  Defaults to 255.</li>
<li>blue:Int - blue component of the color to draw the line with.  Defaults to 255.</li>
</ul>
EndRem
Function ib3d_DrawLine(x1:Float,y1:Float,z1:Float,x2:Float,y2:Float,z2:Float,red:Int=255,green:Int=255,blue:Int=255)
	If Not _g_ib3d_draw_manager Then RuntimeError "iB3D draw manager has not been initialized"
	_g_ib3d_draw_manager.QueueDrawLineCmd(Line3df.CreateFromVals(x1,y1,z1,x2,y2,z2),_SCOLOR(255,red,green,blue))
EndFunction

Rem
bbdoc: Draws a 3d line on the screen from provided instance of line3df
about: <i>Parameters</i>
<ul>
<li>line:Line3df - line to draw</li>
<li>red:Int - red component of the color to draw the line with.  Defaults to 255.</li>
<li>green:Int - green component of the color to draw the line with.  Defaults to 255.</li>
<li>blue:Int - blue component of the color to draw the line with.  Defaults to 255.</li>
</ul>
EndRem
Function ib3d_DrawLine3df(line:Line3df,red:Int=255,green:Int=255,blue:Int=255)
	If Not _g_ib3d_draw_manager Then RuntimeError "iB3D draw manager has not been initialized"
	_g_ib3d_draw_manager.QueueDrawLineCmd(line,_SCOLOR(255,red,green,blue))
EndFunction

Rem
bbdoc: Draws a 3d "wire" triangle on the screen from provided points
about: <i>Parameters</i>
<ul>
<li>x1:Float - X location for point 1 of the triangle</li>
<li>y1:Float - Y location for point 1 of the triangle</li>
<li>z1:Float - Z location for point 1 of the triangle</li>
<li>x2:Float - X location for point 2 of the triangle</li>
<li>y2:Float - Y location for point 2 of the triangle</li>
<li>z2:Float - Z location for point 2 of the triangle</li>
<li>x3:Float - X location for point 3 of the triangle</li>
<li>y3:Float - Y location for point 3 of the triangle</li>
<li>z3:Float - Z location for point 3 of the triangle</li>
<li>red:Int - red component of the color to draw the triangle with.  Defaults to 255.</li>
<li>green:Int - green component of the color to draw the triangle with.  Defaults to 255.</li>
<li>blue:Int - blue component of the color to draw the triangle with.  Defaults to 255.</li>
</ul>
EndRem
Function ib3d_DrawTriangle(x1:Float,y1:Float,z1:Float,x2:Float,y2:Float,z2:Float,x3:Float,y3:Float,z3:Float,red:Int=255,green:Int=255,blue:Int=255)
	If Not _g_ib3d_draw_manager Then RuntimeError "iB3D draw manager has not been initialized"
	_g_ib3d_draw_manager.QueueDrawTriangleCmd(Triangle3df.createFromVals(x1,y1,z1,x2,y2,z2,x3,y3,z3),_SCOLOR(255,red,green,blue))	
EndFunction

Rem
bbdoc: Draws a 3d "wire" triangle on the screen from provided instance of Triangle3df
about: <i>Parameters</i>
<ul>
<li>triangle:Triangle3df - triangle to draw</li>
<li>red:Int - red component of the color to draw the triangle with.  Defaults to 255.</li>
<li>green:Int - green component of the color to draw the triangle with.  Defaults to 255.</li>
<li>blue:Int - blue component of the color to draw the triangle with.  Defaults to 255.</li>
</ul>
EndRem
Function ib3d_DrawTriangle3df(triangle:Triangle3df,red:Int=255,green:Int=255,blue:Int=255)
	If Not _g_ib3d_draw_manager Then RuntimeError "iB3D draw manager has not been initialized"
	_g_ib3d_draw_manager.QueueDrawTriangleCmd(triangle,_SCOLOR(255,red,green,blue))
EndFunction

