Type MESH Extends ib3d_Entity
	Global MeshList:TList=Null
															
	Field _mesh:Irrlicht.Core.IReferenceCounted=Null
	Field _bReadOnly:Int=True

	Method New()
		If Not MeshList Then MeshList=New TList
		MeshList.AddLast(Self)
	EndMethod
	
	Method Destroy()
		Super.Destroy()
		MeshList.Remove(Self)
		_mesh=Null
	EndMethod
	
	Method countSurfaces:Int(frame:Int=0)
		If Not _mesh Or Not _mesh.isValid() Then RuntimeError "Mesh is invalid"
		If IAnimatedMesh(_mesh) Then Return IAnimatedMesh(_mesh).getMesh(frame).getMeshBufferCount()
		If IMesh(_mesh) Then Return IMesh(_mesh).getMeshBufferCount()
	EndMethod

	Method getSurface:SURFACE(index:Int=0,frame:Int=0)
		If Not _mesh Or Not _mesh.isValid() Then RuntimeError "Mesh is invalid"
		Local retval:SURFACE=_g_ib3d_engine.generateSurface()
		
		If IAnimatedMesh(_mesh) Then 
			retval._mbuf=IAnimatedMesh(_mesh).getMesh(frame).getMeshBuffer(index)
			retval._bReadOnly=True
		ElseIf IMesh(_mesh) Then 
			retval._mbuf=IMesh(_mesh).getMeshBuffer(index)
		EndIf
				
		Return retval
	EndMethod

	Rem
   	Method SetColor(red:Float,green:Float,blue:Float,MaterialIDX:Int=0)
		If Not _mesh Or Not _mesh.isValid() Then RuntimeError "Mesh is invalid"
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		Local colorf:SColorf=SColorf.createFromSColor(color)

		' get the # of frames
		Local frameCnt:Int=0
		If IAnimatedMesh(_mesh) Then frameCnt=IAnimatedMesh(_mesh).getFrameCount()	
		
		For Local i:Int=0 Until frameCnt
			Local surfaceCnt:Int=countSurfaces(i)
			For Local j:Int=0 Until surfaceCnt
				Local surf:SURFACE=getSurface(j,i)
				Local vertCnt:Int=surf.getVertexCount()
				For Local k:Int=0 Until vertCnt
					surf.setVertexColor(k,red,green,blue)
				Next
			Next
		Next
		
		'_node.getMaterial(MaterialIDX).getDiffuseColor().set(colorf.getAlpha(),colorf.getRed(),colorf.getGreen(),colorf.getBlue())
   	EndMethod
	EndRem
	
	Method updateNormals()
		If Not _mesh Or Not _mesh.isValid() Then RuntimeError "Mesh is invalid"
		Local mm:IMeshManipulator=_g_ib3d_engine.smgr.getMeshManipulator()
		If IAnimatedMesh(_mesh) Then
			Local frameCnt:Int=IAnimatedMesh(_mesh).getFrameCount()
			For Local i:Int=0 To frameCnt-1
				mm.recalculateNormals(IAnimatedMesh(_mesh).getMesh(i))
			Next
		Else
			mm.recalculateNormals(IMesh(_mesh))
		EndIf
	EndMethod

	Method meshWidth:Int(frame:Int=0)
		If Not _mesh Or Not _mesh.isValid() Then RuntimeError "Mesh is invalid"
		Local extent:Vector3df
		If IAnimatedMesh(_mesh) Then extent=IAnimatedMesh(_mesh).getMesh(frame).getBoundingBox().getExtent()
		If IMesh(_mesh) Then extent=IMesh(_mesh).getBoundingBox().getExtent()
		Return extent.getX()
	EndMethod
		
	Method meshHeight:Int(frame:Int=0)
		If Not _mesh Or Not _mesh.isValid() Then RuntimeError "Mesh is invalid"
		Local extent:Vector3df
		If IAnimatedMesh(_mesh) Then extent=IAnimatedMesh(_mesh).getMesh(frame).getBoundingBox().getExtent()
		If IMesh(_mesh) Then extent=IMesh(_mesh).getBoundingBox().getExtent()
		Return extent.getY()
	EndMethod

	Method meshDepth:Int(frame:Int=0)
		If Not _mesh Or Not _mesh.isValid() Then RuntimeError "Mesh is invalid"
		Local extent:Vector3df
		If IAnimatedMesh(_mesh) Then extent=IAnimatedMesh(_mesh).getMesh(frame).getBoundingBox().getExtent()
		If IMesh(_mesh) Then extent=IMesh(_mesh).getBoundingBox().getExtent()
		Return extent.getZ()
	EndMethod
			
	Method rotateMesh(rx:Float,ry:Float,rz:Float)
		If Not _mesh Or Not _mesh.isValid() Then RuntimeError "Mesh is invalid"
		If IAnimatedMesh(_mesh) Then Throw "Animated meshes are not editable"		

		Local mm:IMeshManipulator=_g_ib3d_engine.smgr.getMeshManipulator()
		If IAnimatedMesh(_mesh)
			_RotateAnimMesh(IAnimatedMesh(_mesh).handle,mm.handle,rx,ry,rz)
		Else
			_RotateMesh(IMesh(_mesh).handle,mm.handle,rx,ry,rz)
		EndIf
	EndMethod

	Method scaleMesh(x_scale:Float,y_scale:Float,z_scale:Float)
		If Not _mesh Or Not _mesh.isValid() Then RuntimeError "Mesh is invalid"
		If IAnimatedMesh(_mesh) Then Throw "Animated meshes are not editable"		
		
		Local mm:IMeshManipulator=_g_ib3d_engine.smgr.getMeshManipulator()
		mm.scaleMesh(SMesh(_mesh),Vector3df.createFromVals(x_scale,y_scale,z_scale))
	EndMethod
	
	Method flipMesh()
		If Not _mesh Or Not _mesh.isValid() Then RuntimeError "Mesh is invalid"
		If IAnimatedMesh(_mesh) Then Throw "Animated meshes are not editable"		
		
		Local mm:IMeshManipulator=_g_ib3d_engine.smgr.getMeshManipulator()
		mm.flipSurfaces(SMesh(_mesh))
	EndMethod
			
EndType

Function ib3d_CreateMesh:MESH(parent:ib3d_Entity=Null)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.createMesh(parent)
EndFunction

Function ib3d_LoadAnimMesh:MESH(file:String,parent:ib3d_Entity=Null)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.loadAnimMesh(file,parent)
EndFunction

Function ib3d_LoadMesh:MESH(file:String,parent:ib3d_Entity=Null)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.loadMesh(file,parent)
EndFunction

Function ib3d_LoadOctTreeMesh:MESH(file:String,parent:ib3d_Entity=Null)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.loadOctTreeMesh(file,parent)
EndFunction

Function ib3d_RotateMesh(mesh:MESH,rx:Float,ry:Float,rz:Float)
	If Not mesh Then Throw "Mesh does not exist" 
	mesh.RotateMesh(rx,ry,rz) 
EndFunction

' TODO: note new frame parameter
Function ib3d_CountSurfaces:Int(mesh:MESH,frame:Int=0)
	If Not mesh Then Throw "Mesh does not exist" 
	Return mesh.countSurfaces(frame)
EndFunction

Function ib3d_MeshWidth:Float(mesh:MESH,frame:Int=0)
	If Not mesh Then Throw "Mesh does not exist" 
	Return mesh.meshWidth(frame)
EndFunction

Function ib3d_MeshHeight:Float(mesh:MESH,frame:Int=0)
	If Not mesh Then Throw "Mesh does not exist" 
	Return mesh.meshHeight(frame)
EndFunction

Function ib3d_MeshDepth:Float(mesh:MESH,frame:Int=0)
	If Not mesh Then Throw "Mesh does not exist" 
	Return mesh.meshDepth(frame)
EndFunction

Function ib3d_UpdateNormals(mesh:MESH)
	If Not mesh Then Throw "Mesh does not exist" 
	mesh.updateNormals()
EndFunction

Function ib3d_ScaleMesh(mesh:MESH,x_scale:Float,y_scale:Float,z_scale:Float)
	If Not mesh Then Throw "Mesh does not exist" 
	mesh.scaleMesh(x_scale,y_scale,z_scale)
EndFunction

Function ib3d_FlipMesh(mesh:MESH)
	If Not mesh Then Throw "Mesh does not exist" 
	mesh.flipMesh()
EndFunction

Function ib3d_GetSurface:SURFACE(mesh:MESH,index:Int=0,frame:Int=0)
	If Not mesh Then Throw "Mesh does not exist" 
	Return mesh.getSurface(index,frame)
EndFunction

Rem

- Meshes

- CreateCube - addCubeSceneNode (f32 size=10.0f, ISceneNode *parent=0, s32 id=-1, const core::vector3df &position=core::vector3df(0, 0, 0), const core::vector3df &rotation=core::vector3df(0, 0, 0), const core::vector3df &scale=core::vector3df(1.0f, 1.0f, 1.0f))=0 

Creates a cube mesh/entity and returns its handle. 

The cube will extend from -1,-1,-1 to +1,+1,+1. 

The optional parent parameter allow you to specify a parent entity for the cube so that when the parent is moved the child cube will move with it. However, this relationship is one way; applying movement commands to the child will not affect the parent. 

Specifying a parent entity will still result in the cube being created at position 0,0,0 rather than at the parent entity's position. 

Creation of cubes, cylinders and cones are a great way of getting scenes set up quickly, as they can act as placeholders for more complex pre-modeled meshes later on in program development. 


- CreateSphere - addSphereSceneNode (f32 radius=5.0f, s32 polyCount=16, ISceneNode *parent=0, s32 id=-1, const core::vector3df &position=core::vector3df(0, 0, 0), const core::vector3df &rotation=core::vector3df(0, 0, 0), const core::vector3df &scale=core::vector3df(1.0f, 1.0f, 1.0f))=0

segments (optional) - sphere detail. Defaults to 8. 
parent (optional) - parent entity of sphere 

Creates a sphere mesh/entity and returns its handle. 

The sphere will be centred at 0,0,0 and will have a radius of 1. 

The segments value must be in the range 2-100 inclusive, although this is only checked in debug mode. A common mistake is to leave debug mode off and specify the parent parameter (usually an eight digit memory address) in the place of the segments value. As the amount of polygons used to create a sphere is exponentially proportional to the segments value, this will result in Blitz trying to create a sphere with unimaginable amounts of polygons! Depending on how unlucky you are, your computer will then crash. 

Example segments values: 
8: 224 polygons - bare minimum amount of polygons for a sphere 
16: 960 polygons - smooth looking sphere at medium-high distances 
32: 3968 polygons - smooth sphere at close distances 

The optional parent parameter allow you to specify a parent entity for the sphere so that when the parent is moved the child sphere will move with it. However, this relationship is one way; applying movement commands to the child will not affect the parent. 

Specifying a parent entity will still result in the sphere being created at position 0,0,0 rather than at the parent entity's position. 



CreateCylinder 
CreateCone 
PaintMesh 
LightMesh 
FitMesh  - getExtent() to find new limits, getExtent() to find existing limit.  get difference and then fit using scale.
PositionMesh 
MeshesIntersect - maybe use bounding boxes?

EndRem
