
Type PRIMITIVE Extends ib3d_Entity
	Global PrimitiveList:TList=Null
							
	Method New() 
		If Not PrimitiveList Then PrimitiveList=New TList
		PrimitiveList.AddLast(Self)
	EndMethod
	
	Method Destroy()
		Super.Destroy()
		PrimitiveList.Remove(Self)
	EndMethod
EndType

Rem
bbdoc: Creates a cube entity
returns: The new cube entity
about: <i>Parameters</i>
<ul>
<li>parent:ib3d_Entity - parent entity of cube</li>
</ul>
EndRem
Function ib3d_CreateCube:PRIMITIVE(parent:ib3d_Entity=Null)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.createCube(parent)
EndFunction

Rem
bbdoc: Creates a sphere entity
returns: The new sphere entity
about: <i>Parameters</i>
<ul>
<li>segments:Int - sphere detail. (Defaults to 8)</li>
<li>parent:ib3d_Entity - parent entity of cube</li>
</ul>
The sphere will be centred at 0,0,0 and will have a radius of 1. <br><br>
The segments value must be in the range 2-100 inclusive, although this is only checked in debug mode. A common mistake is to leave debug mode off and specify the parent parameter (usually an eight digit memory address) in the place of the segments value. As the amount of polygons used to create a sphere is exponentially proportional to the segments value, this will result in Blitz trying to create a sphere with unimaginable amounts of polygons! Depending on how unlucky you are, your computer will then crash. <br><br>
Example segments values: 
8: 224 polygons - bare minimum amount of polygons for a sphere 
16: 960 polygons - smooth looking sphere at medium-high distances 
32: 3968 polygons - smooth sphere at close distances <br><br>
The optional parent parameter allow you to specify a parent entity for the sphere so that when the parent is moved the child sphere will move with it. However, this relationship is one way; applying movement commands to the child will not affect the parent. <br><br>
Specifying a parent entity will still result in the sphere being created at position 0,0,0 rather than at the parent entity's position. 
EndRem
Function ib3d_CreateSphere:PRIMITIVE(segments:Int=8,parent:ib3d_Entity=Null)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.createSphere(segments,parent)
EndFunction

