' ------------------------------------------------------------------------------------
' irr::core::dimension2d<f32>
' ------------------------------------------------------------------------------------

Public

Type Dimension2df Extends IrrBase Final

	'--------------------------------------------
	' operator overloads
	'--------------------------------------------

	Method EqEq:Int(pOther:Dimension2df)
		Return IrrCore_Dimension2df_EqEq(handle,pOther.handle)
	EndMethod
	Method NotEq:Int(pOther:Dimension2df)
		Return IrrCore_Dimension2df_NotEq(handle,pOther.handle)
	EndMethod
	Method Eq(pOther:Dimension2df)
		IrrCore_Dimension2df_Eq(handle,pOther.handle)
	EndMethod

	Method Div:Dimension2df(scale:Float)
		Return Dimension2df.createFromHandle(IrrCore_Dimension2df_Div(handle,scale),True)
	EndMethod
	
	Method Mult:Dimension2df(scale:Float)
		Return Dimension2df.createFromHandle(IrrCore_Dimension2df_Mult(handle,scale),True)
	EndMethod

	Method DivEq:Dimension2df(scale:Float)
		IrrCore_Dimension2df_DivEq(handle, scale)
		Return Self
	EndMethod
	
	Method MultEq:Dimension2df(scale:Float)
		IrrCore_Dimension2df_MultEq(handle, scale)
		Return Self
	EndMethod

	Method Plus:Dimension2df(other:Dimension2df)
		Return Dimension2df.createFromHandle(IrrCore_Dimension2df_Plus(handle, other.handle), True)
	EndMethod
	
	Method PlusEq:Dimension2df(other:Dimension2df)
		IrrCore_Dimension2df_PlusEq(handle, other.handle)
		Return Self
	EndMethod
	
	'--------------------------------------------
	' get/set routines
	'--------------------------------------------
	
	Method getWidth:Float()
		Return IrrCore_Dimension2df_getWidth(handle)
	EndMethod
	Method getHeight:Float()
		Return IrrCore_Dimension2df_getHeight(handle)
	EndMethod
	Method setWidth(width:Float)
		IrrCore_Dimension2df_setWidth(handle,width)
	EndMethod
	Method setHeight(height:Float)
		IrrCore_Dimension2df_setHeight(handle,height)
	EndMethod

	Method Set:Dimension2df(width:Float,height:Float)
		IrrCore_Dimension2df_set(handle,width,height)
		Return Self
	EndMethod
	
	Method getArea:Float()
		Return IrrCore_Dimension2df_getArea(handle) 
	EndMethod

	Method getOptimalSize:Dimension2df(bRequirePowerOfTwo:Int = True, bRequireSquare:Int = False, bLarger:Int = True)
		Return Dimension2df.createFromHandle(IrrCore_Dimension2df_getOptimalSize(handle, bRequirePowerOfTwo, bRequireSquare, bLarger), True)
	EndMethod

	Method getInterpolated:Dimension2df(other:Dimension2df, d:Float)
		Return Dimension2df.createFromHandle(IrrCore_Dimension2df_getInterpolated(handle, other.handle, d))
	EndMethod
	
	'--------------------------------------------
	' create/delete routines
	'--------------------------------------------

	Function create:Dimension2df(x:Float=0,y:Float=0)
		Return createFromHandle(IrrCore_Dimension2df_new(x,y))
	EndFunction
	
	Function createFromPos:Dimension2df(pSize:Dimension2df)
		Return createFromHandle(IrrCore_Dimension2df_newFromSize(pSize.handle))
	EndFunction
	
	Function createFromHandle:Dimension2df(pPos:Int,bMustDel:Int=True)
		If Not pPos Then Return Null
		Local oRetVal:Dimension2df=New Dimension2df
		oRetVal.attach(pPos,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrCore_Dimension2df_del(handle)
	EndMethod	
EndType

