' ------------------------------------------------------------------------------------
' irr::core::line3d<f32> wrappers
' ------------------------------------------------------------------------------------
Public

Type Line3df Extends IrrBase Final

	'--------------------------------------------
	' operator overloads
	'--------------------------------------------

	Method Plus:Line3df(pPoint:Vector3df)
		Return createFromHandle(IrrCore_Line3df_Plus(handle,pPoint.handle))
	EndMethod
	Method Minus:Line3df(pPoint:Vector3df)
		Return createFromHandle(IrrCore_Line3df_Minus(handle,pPoint.handle))
	EndMethod
	Method PlusEq(pPoint:Vector3df)
		IrrCore_Line3df_PlusEq(handle,pPoint.handle)
	EndMethod
	Method MinusEq(pPoint:Vector3df)
		IrrCore_Line3df_PlusEq(handle,pPoint.handle)
	EndMethod
	Method EqEq:Int(pLine2:Line3df)
		Return IrrCore_Line3df_EqEq(handle,pLine2.handle)
	EndMethod
	Method NotEq:Int(pLine2:Line3df)
		Return IrrCore_Line3df_NotEq(handle,pLine2.handle)
	EndMethod

	'--------------------------------------------
	' get/set routines
	'--------------------------------------------
	
	Method setLineFromVals(xa:Float,ya:Float,za:Float,xb:Float,yb:Float,zb:Float)
		IrrCore_Line3df_setLineFromVals(handle,xa,ya,za,xb,yb,zb)
	EndMethod
	
	Method setLineFromVects(pStart:Vector3df,pEnd:Vector3df)
		IrrCore_Line3df_setLineFromVects(handle,pStart.handle,pEnd.handle)
	EndMethod
	
	Method setLineFromLine(pLine2:Line3df)
		IrrCore_Line3df_setLineFromLine(handle,pLine2.handle)
	EndMethod

	Method getLength:Float()
		Return IrrCore_Line3df_getLength(handle)
	EndMethod
	Method getLengthSQ:Float()
		Return IrrCore_Line3df_getLengthSQ(handle)
	EndMethod

	Method getMiddle:Vector3df()
		Return Vector3df.createFromHandle(IrrCore_Line3df_getMiddle(handle))
	EndMethod
	
	Method getVector:Vector3df()
		Return Vector3df.createFromHandle(IrrCore_Line3df_getVector(handle))
	EndMethod
	Method isPointBetweenStartAndEnd:Int(pPoint:Vector3df)
		Return IrrCore_Line3df_isPointBetweenStartAndEnd(handle,pPoint.handle)
	EndMethod

	Method getClosestPoint:Vector3df(pPoint:Vector3df)
		Return Vector3df.createFromHandle(IrrCore_Line3df_getClosestPoint(handle,pPoint.handle))
	EndMethod
	
	Method getStart:Vector3df()
		' set to not delete
		Return Vector3df.createFromHandle(IrrCore_Line3df_getStart(handle),False)
	EndMethod
	Method getEnd:Vector3df()
		' set to not delete
		Return Vector3df.createFromHandle(IrrCore_Line3df_getEnd(handle),False)
	EndMethod

	Method getIntersectionWithSphere:Int(pSorigin:Vector3df,sradius:Float,pOutdistance:Double Ptr)
		Return IrrCore_Line3df_getIntersectionWithSphere(handle,pSorigin.handle,sradius,pOutdistance)
	EndMethod

	'--------------------------------------------
	' create/delete routines
	'--------------------------------------------
	
	Function createFromVals:Line3df(xa:Float,ya:Float,za:Float,xb:Float,yb:Float,zb:Float)
		Return createFromHandle(IrrCore_Line3df_newFromVals(xa,ya,za,xb,yb,zb))
	EndFunction

	Function createFromVects:Line3df(pStart:Vector3df,pEnd:Vector3df)
		Return createFromHandle(IrrCore_Line3df_newFromVects(pStart.handle,pEnd.handle))
	EndFunction

	Function createFromLine:Line3df(pOther:Line3df)
		Return createFromHandle(IrrCore_Line3df_newFromLine(pOther.handle))
	EndFunction

	Function create:Line3df()
		Return createFromHandle(IrrCore_Line3df_new())
	EndFunction
	
	Function createFromHandle:Line3df(pLine:Int,bMustDel:Int=True)
		If Not pLine Then Return Null
		Local oRetVal:Line3df=New Line3df
		oRetVal.attach(pLine,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrCore_Line3df_del(handle)
	EndMethod	
EndType

