' ------------------------------------------------------------------------------------
' irr::core::vector3di wrappers
' ------------------------------------------------------------------------------------

Public

Type Vector3di Extends IrrBase Final

	'--------------------------------------------
	' misc routines
	'--------------------------------------------

	Method isBetweenPoints:Int(pBeginVect:Vector3di,pEndVect:Vector3di)
		Return IrrCore_Vector3di_isBetweenPoints(handle,pBeginVect.handle,pEndVect.handle) 
	EndMethod

	Method invert:Vector3di()
		IrrCore_Vector3di_invert(handle)
		Return Self
	EndMethod

	Method setLength:Vector3di(newlength:Int)
		IrrCore_Vector3di_setLength(handle,newlength)
    Return Self
	EndMethod

	Method crossProduct:Vector3di(pOtherVect:Vector3di)
		Return createFromHandle(IrrCore_Vector3di_crossProduct(handle,pOtherVect.handle))
	EndMethod

	Method getInterpolated:Vector3di(pOtherVect:Vector3di,d:Double)
		Return createFromHandle(IrrCore_Vector3di_getInterpolated(handle,pOtherVect.handle,d))
	EndMethod

	Method rotateXZBy(degrees:Double,pCenter:Vector3di = Null)
		IrrCore_Vector3di_rotateXZBy(handle,degrees,_HANDLE(pCenter))
	EndMethod

	Method rotateXYBy(degrees:Double,pCenter:Vector3di = Null)
		IrrCore_Vector3di_rotateXYBy(handle,degrees,_HANDLE(pCenter))
	EndMethod

	Method rotateYZBy(degrees:Double,pCenter:Vector3di = Null)
		IrrCore_Vector3di_rotateYZBy(handle,degrees,_HANDLE(pCenter))
	EndMethod

	Method normalize:Vector3di()
		Return createFromHandle(IrrCore_Vector3di_normalize(handle),False)
	EndMethod
	
	Method getDistanceFrom:Int(pOther:Vector3di)
		Return IrrCore_Vector3di_getDistanceFrom(handle,pOther.handle)
	EndMethod

	Method getDistanceFromSQ:Int(pOther:Vector3di)
		Return IrrCore_Vector3di_getDistanceFromSQ(handle,pOther.handle)
	EndMethod

	Method dotProduct:Int(pOther:Vector3di)
		Return IrrCore_Vector3di_dotProduct(handle,pOther.handle)
	EndMethod

	Method getLength:Double()
		Return IrrCore_Vector3di_getLength(handle)
	EndMethod

	Method getLengthSQ:Int()
		Return IrrCore_Vector3di_getLengthSQ(handle)
	EndMethod

	Method getAs4Values(vals:Int Ptr)
		IrrCore_Vector3di_getAs4Values(handle,vals)
	EndMethod

	Method getHorizontalAngle:Vector3di()
		Return Vector3di.createFromHandle(IrrCore_Vector3di_getHorizontalAngle(handle),True)
	EndMethod

	Method Equals:Int(pOther:Vector3di,tolerance:Int = 0.00005)
		Return IrrCore_Vector3di_Equals(handle,pOther.handle,tolerance)
	EndMethod

	Method getInterpolated_quadratic:Vector3di(v2:Vector3di,v3:Vector3di,d:Double)
		Return Vector3di.createFromHandle(IrrCore_Vector3di_getInterpolated_quadratic(handle,v2.handle,v3.handle,d),True)
	EndMethod

	Method interpolate:Vector3di(a:Vector3di, b:Vector3di, d:Double)
		IrrCore_Vector3di_interpolate(handle, a.handle, b.handle, d)
		Return Self
	EndMethod
	
	Method rotationToDirection:Vector3di(forwards:Vector3di = Null)
		Return Vector3di.createFromHandle(IrrCore_Vector3di_rotationToDirection(handle, _HANDLE(forwards)), True)
	EndMethod

	'--------------------------------------------
	' operator overloads
	'--------------------------------------------
	
	Method setEq:Vector3di(pOther:Vector3di)
		IrrCore_Vector3di_setEq(handle,pOther.handle)
		Return Self
	EndMethod	

	Method EqEq:Int(pOther:Vector3di)
		Return IrrCore_Vector3di_EqEq(handle,pOther.handle)
	EndMethod

	Method NotEq:Int(pOther:Vector3di)
		Return IrrCore_Vector3di_NotEq(handle,pOther.handle)
	EndMethod
	
	Method GreatEq(pOther:Vector3di)
		IrrCore_Vector3di_GreatEq(handle,pOther.handle)
	EndMethod

	Method LessEq(pOther:Vector3di)
		IrrCore_Vector3di_LessEq(handle,pOther.handle)
	EndMethod

	Method Plus:Vector3di(pOther:Vector3di)		
		Return createFromHandle(IrrCore_Vector3di_Plus(handle,pOther.handle))
	EndMethod
	
	Method PlusEq:Vector3di(pOther:Vector3di)
		IrrCore_Vector3di_PlusEq(handle,pOther.handle)
		Return Self
	EndMethod
	
	Method PlusVal:Vector3di(val:Int)		
		Return createFromHandle(IrrCore_Vector3di_PlusVal(handle, val))
	EndMethod
	
	Method PlusEqVal:Vector3di(val:Int)
		IrrCore_Vector3di_PlusEqVal(handle, val)
		Return Self
	EndMethod

	Method Minus:Vector3di(pOther:Vector3di)		
		Return createFromHandle(IrrCore_Vector3di_Minus(handle,pOther.handle))
	EndMethod
	
	Method MinusEq:Vector3di(pOther:Vector3di)		
		IrrCore_Vector3di_MinusEq(handle,pOther.handle)
		Return Self
	EndMethod
	
	Method MinusVal:Vector3di(val:Int)		
		Return createFromHandle(IrrCore_Vector3di_MinusVal(handle, val))
	EndMethod
	
	Method MinusEqVal:Vector3di(val:Int)		
		IrrCore_Vector3di_MinusEqVal(handle, val)
		Return Self
	EndMethod

	Method MultVect:Vector3di(pOther:Vector3di)
		Return createFromHandle(IrrCore_Vector3di_MultVect(handle,pOther.handle))
	EndMethod
	
	Method Mult:Vector3di(v:Int)
		Return createFromHandle(IrrCore_Vector3di_Mult(handle,v))
	EndMethod

	Method MultEqVect:Vector3di(pOther:Vector3di)
		IrrCore_Vector3di_MultEqVect(handle,pOther.handle)
		Return Self
	EndMethod
	
	Method MultEq:Vector3di(v:Int)
		IrrCore_Vector3di_MultEq(handle,v)
		Return Self
	EndMethod
	
	Method DivVect:Vector3di(pOther:Vector3di)
		Return createFromHandle(IrrCore_Vector3di_DivVect(handle,pOther.handle))
	EndMethod

	Method Div:Vector3di(v:Int)
		Return createFromHandle(IrrCore_Vector3di_Div(handle,v))
	EndMethod

	Method DivEqVect:Vector3di(pOther:Vector3di)
		IrrCore_Vector3di_DivEqVect(handle,pOther.handle)
		Return Self
	EndMethod
	
	Method DivEq:Vector3di(v:Int)
		IrrCore_Vector3di_DivEq(handle,v)
		Return Self
	EndMethod

	Method Less:Int(other:Vector3di)
		Return IrrCore_Vector3di_Less(handle,other.handle)	
	EndMethod
	
	Method Great:Int(other:Vector3di)
		Return IrrCore_Vector3di_Great(handle,other.handle)
	EndMethod
	
	'--------------------------------------------
	' set routines
	'--------------------------------------------

	Method setX(x:Int)
		IrrCore_Vector3di_setX(handle,x)
	EndMethod
	
	Method setY(y:Int)
		IrrCore_Vector3di_setY(handle,y)
	EndMethod
	
	Method setZ(z:Int)
		IrrCore_Vector3di_setZ(handle,z)
	EndMethod

	Method setFromVect:Vector3di(pOther:Vector3di)
		IrrCore_Vector3di_setFromVect(handle,pOther.handle)
		Return Self
	EndMethod

	Method set:Vector3di(x:Int,y:Int,z:Int)
		IrrCore_Vector3di_set(handle,x,y,z)
		Return Self
	EndMethod

	'--------------------------------------------
	' get routines
	'--------------------------------------------

	Method getX:Int()
		Return IrrCore_Vector3di_getX(handle)
	EndMethod
	Method getY:Int()
		Return IrrCore_Vector3di_getY(handle)
	EndMethod
	Method getZ:Int()
		Return IrrCore_Vector3di_getZ(handle)
	EndMethod

	'--------------------------------------------
	' create/delete routines
	'--------------------------------------------

	Function create:Vector3di()
		Return createFromHandle(IrrCore_Vector3di_new())
	EndFunction

	Function createFromVals:Vector3di(x:Int,y:Int,z:Int)
		Return createFromHandle(IrrCore_Vector3di_new(x,y,z))
	EndFunction

	Function createFromVal:Vector3di(n:Int)
		Return createFromHandle(IrrCore_Vector3di_newFromVal(n))
	EndFunction

	Function createFromVect:Vector3di(vect:Vector3di)
		Return createFromHandle(IrrCore_Vector3di_newFromVect(vect.handle))
	EndFunction

	Function createFromHandle:Vector3di(pVect:Int,bMustDel:Int=True)
		If Not pVect Then Return Null
		Local oRetVal:Vector3di=New Vector3di
		oRetVal.attach(pVect,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrCore_Vector3di_del(handle)
	EndMethod


EndType
