'-------------------------------------------------
' IAttributes wrapper
'-------------------------------------------------

Public

Type IAttributes Extends IReferenceCounted Final

	Function createFromHandle:IAttributes(pHandle:Int,bMustDel:Int=True)
		If Not pHandle Then Return Null
		Local oRetVal:IAttributes=New IAttributes
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	'! Removes all attributes
	Method clear()
		Irr_IAttributes_clear(handle)
	EndMethod
	
	'! Adds an attribute as integer
	Method addInt(attributeName:String,value:Int)
		Irr_IAttributes_addInt(handle,attributeName,value)
	EndMethod
		
	'! Adds an attribute as float
	Method addFloat(attributeName:String,value:Float)
		Irr_IAttributes_addFloat(handle,attributeName,value)
	EndMethod
	
	'! Adds an attribute as string
	Method addString(attributeName:String,value:String)
		Irr_IAttributes_addString(handle,attributeName,value)
	EndMethod
		
	'! Adds an attribute as bool
	Method addBool(attributeName:String,bValue:Int)
		Irr_IAttributes_addBool(handle,attributeName,bValue)
	EndMethod
	
	'! Adds an attribute as enum
	Method addStringEnum(attributeName:String,enumValue:String,enumerationLiterals:String[])
		Local length:Int=enumerationLiterals.Dimensions()[0]		
		Local temp:Byte Ptr[length]
		
		' convert all strings to ptrs
		For Local i:Int=0 To length-1
			temp[i]=enumerationLiterals[i].ToCString()
		Next
		
		Irr_IAttributes_addStringEnum(handle,attributeName,enumValue,temp)

		' clean up
		For Local i:Int=0 To length-1
			Memfree(temp[i])
		Next
	EndMethod
	
	'! Adds an attribute as enum
	Method addIntEnum(attributeName:String,enumValue:Int,enumerationLiterals:String[])
		Local length:Int=enumerationLiterals.Dimensions()[0]		
		Local temp:Byte Ptr[length]
		
		' convert all strings to ptrs
		For Local i:Int=0 To length-1
			temp[i]=enumerationLiterals[i].ToCString()
		Next
		
		Irr_IAttributes_addIntEnum(handle,attributeName,enumValue,temp)

		' clean up
		For Local i:Int=0 To length-1
			Memfree(temp[i])
		Next
	EndMethod

	'! Adds an attribute as color
	Method addColor(attributeName:String,color:SColor)
		Irr_IAttributes_addColor(handle,attributeName,color.handle)
	EndMethod
	
	'! Adds an attribute as floating point color
	Method addColorf(attributeName:String,color:SColorf)
		Irr_IAttributes_addColorf(handle,attributeName,color.handle)
	EndMethod
	
	'! Adds an attribute as 3d vector
	Method addVector3df(attributeName:String,vector:Vector3df)
		Irr_IAttributes_addVector3df(handle,attributeName,vector.handle)
	EndMethod
	
	'! Adds an attribute as 2d position
	Method addPosition2di(attributeName:String,vector:Position2di)
		Irr_IAttributes_addPosition2di(handle,attributeName,vector.handle)
	EndMethod

	'! Adds an attribute as binary data
	Method addBinary(attributeName:String,data:Byte Ptr,dataSizeInBytes:Int)
		Irr_IAttributes_addBinary(handle,attributeName,data,dataSizeInBytes)
	EndMethod

	'! Adds an attribute as texture reference
	Method addTexture(attributeName:String,texture:ITexture)
		Irr_IAttributes_addTexture(handle,attributeName,texture.handle)
	EndMethod
	
	'! Returns amount of attributes in this collection of attributes.
	Method getAttributeCount:Int()
		Return Irr_IAttributes_getAttributeCount(handle)
	EndMethod
	
	'! Returns attribute name by index. 
	'! \param index: Index value, must be between 0 and getAttributeCount()-1.
	Method getAttributeName:String(index:Int)
		Return Irr_IAttributes_getAttributeName(handle,index)
	EndMethod
	
	'! Returns the type of an attribute
	'! \param attributeName: Name for the attribute
	Method getAttributeTypeByName:Int(attributeName:String)
		Return Irr_IAttributes_getAttributeTypeByName(handle,attributeName)
	EndMethod
	
	'! Returns attribute type by index. 
	'! \param index: Index value, must be between 0 and getAttributeCount()-1.
	Method getAttrbuteTypeByIndex:Int(index:Int)
		Return Irr_IAttributes_getAttributeTypeByIndex(handle,index)
	EndMethod
	
	'! Sets an attribute value as string. 
	'! \param attributeName: Name for the attribute
	'! \param value: Value for the attribute. Set this to 0 to delete the attribute
	Method setStringAttributeByName(attributeName:String,value:String)
		Irr_IAttributes_setStringAttributeByName(handle,attributeName,value)
	EndMethod
	
	'! Gets an attribute as string.
	'! \param attributeName: Name of the attribute to get.
	'! \return Returns value of the attribute previously set by setAttribute() 
	'! or 0 if attribute is not set.
	Method getAttributeAsStringByName:String(attributeName:String)
		Return Irr_IAttributes_getAttributeAsStringByName(handle,attributeName)
	EndMethod
	
	'! Returns attribute value as string by index. 
	'! \param index: Index value, must be between 0 and getAttributeCount()-1.
	Method getAttributeAsStringByIndex:String(index:Int)
		Return Irr_IAttributes_getAttributeAsStringByIndex(handle,index)	
	EndMethod
	
	'! Sets an attribute as boolean value
	Method setBoolAttributeByName(attributeName:String,bValue:Int)
		Irr_IAttributes_setBoolAttributeByName(handle,attributeName,bValue)
	EndMethod
	
	'! Gets an attribute as boolean value
	'! \param attributeName: Name of the attribute to get.
	'! \return Returns value of the attribute previously set by setAttribute()
	Method getAttributeAsBoolByName:Int(attributeName:String)
		Return Irr_IAttributes_getAttributeAsBoolByName(handle,attributeName)
	EndMethod
	
    '! Gets an attribute as boolean value
	'! \param index: Index value, must be between 0 and getAttributeCount()-1
	Method getAttributeAsBoolByIndex:Int(index:Int)
		Return Irr_IAttributes_getAttributeAsBoolByIndex(handle,index)
	EndMethod
	
	'! Sets an attribute as integer value
	Method setIntAttributeByName(attributeName:String,value:Int)
		Irr_IAttributes_setIntAttributeByName(handle,attributeName,value)
	EndMethod
		
	'! Gets an attribute as integer value
	'! \param attributeName: Name of the attribute to get.
	'! \return Returns value of the attribute previously set by setAttribute()
	Method getAttributeAsIntByName:Int(attributeName:String)
		Return Irr_IAttributes_getAttributeAsIntByName(handle,attributeName)
	EndMethod
	
	'! Gets an attribute as integer value
	'! \param index: Index value, must be between 0 and getAttributeCount()-1
	Method getAttributeAsIntByIndex:Int(index:Int)
		Return Irr_IAttributes_getAttributeAsIntByIndex(handle,index)
	ENdMethod
	
	'! Sets a attribute as float value
	Method setFloatAttributeByName(attributeName:String,value:Float)
		Irr_IAttributes_setFloatAttributeByName(handle,attributeName,value)
	EndMethod
	
	'! Gets an attribute as float value
	'! \param attributeName: Name of the attribute to get.
	'! \return Returns value of the attribute previously set by setAttribute()
	Method getAttributeAsFloatByName:Float(attributeName:String)
		Return Irr_IAttributes_getAttributeAsFloatByName(handle,attributeName)
	EndMethod
	
	'! Gets an attribute as float value
	'! \param index: Index value, must be between 0 and getAttributeCount()-1
	Method getAttributeAsFloatByIndex:Float(index:Int)
		Return Irr_IAttributes_getAttributeAsFloatByIndex(handle,index)
	EndMethod
	
	'! Sets a attribute as color
	Method setColorAttributeByName(attributeName:String,color:SColor)
		Irr_IAttributes_setColorAttributeByName(handle,attributeName,color.handle)
	EndMethod
	
	'! Gets an attribute as color
	'! \param attributeName: Name of the attribute to get.
	'! \return Returns value of the attribute previously set by setAttribute()
	Method getAttributeAsColorByName:SColor(attributeName:String)
		Local color:Int=Irr_IAttributes_getAttributeAsColorByName:Int(handle,attributeName)
		If color>0 Then Return SColor.createFromHandle(color,True) Else Return Null
	EndMethod
	
	'! Gets an attribute as color
	'! \param index: Index value, must be between 0 and getAttributeCount()-1
	' NOTE: must delete ptr
	Method getAttributeAsColorByIndex:SColor(index:Int)
		Local color:Int=Irr_IAttributes_getAttributeAsColorByIndex(handle,index)
		If color>0 Then Return SColor.createFromHandle(color,True) Else Return Null
	EndMethod
	
	'! Sets a attribute as floating point color
	Method setSColorfAttributeByName(attributeName:String,color:SColorf)
		Irr_IAttributes_setSColorfAttributeByName(handle,attributeName,color.handle)
	EndMethod
	
	'! Gets an attribute as floating point color
	'! \param attributeName: Name of the attribute to get.
	'! \return Returns value of the attribute previously set by setAttribute()
	Method getAttributeAsColorfByName:SColorf(attributeName:String)
		Local color:Int=Irr_IAttributes_getAttributeAsColorfByName:Int(handle,attributeName)
		If color>0 Then Return SColorf.createFromHandle(color,True) Else Return Null
	EndMethod
	
	'! Gets an attribute as floating point color
	'! \param index: Index value, must be between 0 and getAttributeCount()-1
	Method getAttributeColorfByIndex:SColorf(index:Int)
		Local color:Int=Irr_IAttributes_getAttributeAsColorfByIndex(handle,index)
		If color>0 Then Return SColorf.createFromHandle(color,True) Else Return Null
	EndMethod
	
	'! Sets a attribute as vector
	Method setVector3dfAttributeByName(attributeName:String,vector:Vector3df)
		Irr_IAttributes_setVector3dfAttributeByName(handle,attributeName,vector.handle)
	EndMethod
	
	'! Gets an attribute as floating point color
	'! \param attributeName: Name of the attribute to get.
	'! \return Returns value of the attribute previously set by setAttribute()
	Method getAttributeAsVector3dfByName:Vector3df(attributeName:String)
		Local vector:Int=Irr_IAttributes_getAttributeAsVector3dfByName:Int(handle,attributeName)
		If vector>0 Then Return vector3df.createFromHandle(vector,True) Else Return Null
	EndMethod
	
	'! Gets an attribute as floating point color
	'! \param index: Index value, must be between 0 and getAttributeCount()-1.
	Method getAttributeAsVector3dfByIndex:Vector3df(index:Int)
		Local vector:Int=Irr_IAttributes_getAttributeAsVector3dfByIndex(handle,index)
		If vector>0 Then Return vector3df.createFromHandle(vector,True) Else Return Null
	EndMethod
	
	'! Sets a attribute as position
	Method setPosition2diAttributeByName(attributeName:String,position:Position2di)
		Irr_IAttributes_setPosition2diAttributeByName(handle,attributeName,position.handle)
	EndMethod
	
	Method getAttributeAsPosition2diByName:Position2di(attributeName:String)
		Local position:Int=Irr_IAttributes_getAttributeAsPosition2diByName:Int(handle,attributeName)
		If position>0 Then Return position2di.createFromHandle(position,True) Else Return Null
	EndMethod
	
	Method getAttributeAsPosition2diByIndex:Position2di(index:Int)
		Local position:Int=Irr_IAttributes_getAttributeAsPosition2diByIndex(handle,index)
		If position>0 Then Return position2di.createFromHandle(position,True) Else Return Null
	EndMethod

	'! Sets an attribute as binary data
	Method setBinaryAttributeByName(attributeName:String,data:Byte Ptr,dataSizeInBytes:Int)
		Irr_IAttributes_setBinaryAttributeByName(handle,attributeName,data,dataSizeInBytes)
	EndMethod
	
	'! Gets an attribute as binary data
	'! \param attributeName: Name of the attribute to get.
	Method getAttributeAsBinaryDataByName(attributeName:String,outData:Byte Ptr,maxSizeInBytes:Int)
		Irr_IAttributes_getAttributeAsBinaryDataByName(handle,attributeName,outData,maxSizeInBytes)
	EndMethod
	
	'! Gets an attribute as binary data
	'! \param index: Index value, must be between 0 and getAttributeCount()-1
	Method getAttributeAsBinaryDataByIndex(index:Int,outData:Byte Ptr,maxSizeInBytes:Int)
		Irr_IAttributes_getAttributeAsBinaryDataByIndex(handle,index,outData,maxSizeInBytes)
	EndMethod
	
	'! Sets an attribute as enumeration
	Method setEnumAttributeByName(attributeName:String,enumValue:String,enumerationLiterals:String[])
		Local length:Int=enumerationLiterals.Dimensions()[0]		
		Local temp:Byte Ptr[length]
		
		' convert all strings to ptrs
		For Local i:Int=0 To length-1
			temp[i]=enumerationLiterals[i].ToCString()
		Next
		
		Irr_IAttributes_setEnumAttributeByName(handle,attributeName,enumValue,temp)

		' clean up
		For Local i:Int=0 To length-1
			Memfree(temp[i])
		Next	
	EndMethod
	
	'! Gets an attribute as enumeration
	'! \param attributeName: Name of the attribute to get.
	'! \return Returns value of the attribute previously set by setAttribute()
	Method getAttributeAsStringEnumerationByName:String(attributeName:String)
		Return Irr_IAttributes_getAttributeAsStringEnumerationByName(handle,attributeName)
	EndMethod
	
	'! Gets an attribute as enumeration
	'! \param attributeName: Name of the attribute to get.
	'! \param enumerationLiteralsToUse: Use these enumeration literals to get the index value instead of the set ones.
	'! This is useful when the attribute list maybe was read from an xml file, and only contains the enumeration string, but
	'! no information about its index.
	'! \return Returns value of the attribute previously set by setAttribute()
	Method getAttributeAsIntEnumerationByName:Int(attributeName:String,enumerationLiteralsToUse:String[])
		Local length:Int=enumerationLiteralsToUse.Dimensions()[0]		
		Local temp:Byte Ptr[length]
		
		' convert all strings to ptrs
		For Local i:Int=0 To length-1
			temp[i]=enumerationLiteralsToUse[i].ToCString()
		Next
		
		Local retval:Int=Irr_IAttributes_getAttributeAsIntEnumerationByName(handle,attributeName,temp)

		' clean up
		For Local i:Int=0 To length-1
			Memfree(temp[i])
		Next
		
		Return retval
	EndMethod
	
	'! Gets an attribute as enumeration
	'! \param index: Index value, must be between 0 and getAttributeCount()-1.
	Method getAttributeAsIntEnumerationByIndex:Int(index:Int,enumerationLiteralsToUse:String[])
		Local length:Int=enumerationLiteralsToUse.Dimensions()[0]		
		Local temp:Byte Ptr[length]
		
		' convert all strings to ptrs
		For Local i:Int=0 To length-1
			temp[i]=enumerationLiteralsToUse[i].ToCString()
		Next
		
		Local retval:Int=Irr_IAttributes_getAttributeAsIntEnumerationByIndex(handle,index,temp)

		' clean up
		For Local i:Int=0 To length-1
			Memfree(temp[i])
		Next
		
		Return retval
	EndMethod
	
	'! Gets an attribute as enumeration
	'! \param index: Index value, must be between 0 and getAttributeCount()-1.
	Method getAttributeAsStringEnumerationByIndex:String(index:Int)
		Return Irr_IAttributes_getAttributeAsStringEnumerationByIndex(handle,index)
	EndMethod
	
	'! Gets the list of enumeration literals of an enumeration attribute 
	'! \param attributeName: Name of the attribute to get.
	Method getAttributeEnumerationLiteralsOfEnumerationByName:String[](attributeName:String)		
		' temp storage for up to 50 enumerations
		Local literals:StringC[50]		
		Local stringc_param:Int[50]		
		
		For Local i:Int=0 To 49
			literals[i]=stringc.create()
			stringc_param[i]=literals[i].handle
		Next

		Local literalCnt:Int=Irr_IAttributes_getAttributeEnumerationLiteralsOfEnumerationByName(handle,attributeName,stringc_param)
		
		Local outLiterals:String[literalCnt]
		For Local i:Int=0 To literalCnt-1
			outLiterals[i]=literals[i].getString()
		Next
		
		Return outLiterals
	EndMethod
	
	'! Gets the list of enumeration literals of an enumeration attribute 
	'! \param index: Index value, must be between 0 and getAttributeCount()-1.
	Method getAttributeEnumerationLiteralsOfEnumerationByIndex:String[](index:Int)
		' temp storage for up to 50 enumerations
		Local literals:StringC[50]		
		Local stringc_param:Int[50]		
		
		For Local i:Int=0 To 49
			literals[i]=stringc.create()
			stringc_param[i]=literals[i].handle
		Next

		Local literalCnt:Int=Irr_IAttributes_getAttributeEnumerationLiteralsOfEnumerationByIndex(handle,index,stringc_param)
		
		Local outLiterals:String[literalCnt]
		For Local i:Int=0 To literalCnt-1
			outLiterals[i]=literals[i].getString()
		Next
		
		Return outLiterals
	EndMethod
	
	'! Sets an attribute as texture reference
	Method setTextureAttributeByName(attributeName:String,texture:ITexture)
		Irr_IAttributes_setTextureAttributeByName(handle,attributeName,texture.handle)
	EndMethod
	
	'! Gets an attribute as texture reference
	'! \param attributeName: Name of the attribute to get.
	Method getAttributeAsTextureByName:ITexture(attributeName:String)
		Local texture:Int=Irr_IAttributes_getAttributeAsTextureByName(handle,attributeName)
		If texture>0 Then Return ITexture.createFromHandle(texture,True) Else Return Null
	EndMethod
	
	'! Gets an attribute as texture reference
	'! \param index: Index value, must be between 0 and getAttributeCount()-1.
	Method getAttributeAsTextureByIndex:ITexture(index:Int)
		Local texture:Int=Irr_IAttributes_getAttributeAsTextureByIndex(handle,index)
		If texture>0 Then Return ITexture.createFromHandle(texture,True) Else Return Null
	EndMethod
	
	'! Returns if an attribute with a name exists
	Method existsAttribute:Int(attributeName:String)
		Return Irr_IAttributes_existsAttribute(handle,attributeName)
	EndMethod
	
	'! Returns attribute index from name, -1 if not found
	Method findAttribute:Int(attributeName:String)
		Return Irr_IAttributes_findAttribute:Int(handle,attributeName)
	EndMethod
	
	'! Sets an attribute value as string. 
	'! \param attributeName: Name for the attribute
	Method setStringAttributeByIndex(index:Int,value:String)
		Irr_IAttributes_setStringAttributeByIndex(handle,index,value)
	EndMethod
	
	'! Sets an attribute as boolean value
	Method setBoolAttributeByIndex(index:Int,bValue:Int)
		Irr_IAttributes_setBoolAttributeByIndex(handle,index,bValue)
	EndMethod
	
	'! Sets an attribute as integer value
	Method setIntAttributeByIndex(index:Int,value:Int)
		Irr_IAttributes_setIntAttributeByIndex(handle,index,value)
	EndMethod
	
	'! Sets an attribute as float value
	Method setFloatAttributeByIndex(index:Int,value:Float)
		Irr_IAttributes_setFloatAttributeByIndex(handle,index,value)
	EndMethod 
	
	'! Sets an attribute as color
	Method setColorAttributeByIndex(index:Int,color:SColor)
		Irr_IAttributes_setColorAttributeByIndex(handle,index,color.handle)
	EndMethod
	
	'! Sets an attribute as floating point color
	Method setColorfAttributeByIndex(index:Int,color:SColorf)
		Irr_IAttributes_setColorfAttributeByIndex(handle,index,color.handle)
	EndMethod
	
	'! Sets an attribute as vector
	Method setVector3dfAttributeByIndex(index:Int,vector:Vector3df)
		Irr_IAttributes_setVector3dfAttributeByIndex(handle,index,vector.handle)
	EndMethod
	
	'! Sets an attribute as binary data
	Method setBinaryAttributeByIndex(index:Int,data:Byte Ptr,dataSizeInBytes:Int)
		Irr_IAttributes_setBinaryAttributeByIndex(handle,index,data,dataSizeInBytes)
	EndMethod
	
	'! Sets an attribute as enumeration
	Method setEnumAttributeByIndex(index:Int,enumValue:String,enumerationLiterals:String[])
		Local length:Int=enumerationLiterals.Dimensions()[0]		
		Local temp:Byte Ptr[length]
		
		' convert all strings to ptrs
		For Local i:Int=0 To length-1
			temp[i]=enumerationLiterals[i].ToCString()
		Next
		
		Irr_IAttributes_setEnumAttributeByIndex(handle,index,enumValue,temp)

		' clean up
		For Local i:Int=0 To length-1
			Memfree(temp[i])
		Next	
	EndMethod
	
	'! Sets an attribute as texture reference
	Method setTextureAttributeByIndex(index:Int,texture:ITexture)
		Irr_IAttributes_setTextureAttributeByIndex(handle,index,texture.handle)
	EndMethod

	Rem
		rect2di attribute
	EndRem

	Method addRect(attributeName:String,rect:Recti)
		Irr_IAttributes_addRect(handle,attributeName,rect.handle)
	EndMethod
	
	Method setRect3diAttributeByName(attributeName:String,rect:Recti)
		Irr_IAttributes_setRect3diAttributeByName(handle,attributeName,rect.handle)
	EndMethod
	
	Method getAttributeAsRect:Recti(attributeName:String)
		Return Recti.createFromHandle(Irr_IAttributes_getAttributeAsRect(handle,attributeName),True)
	EndMethod
	
	Method getAttributeAsRectByIndex:Recti(index:Int)
		Return Recti.createFromHandle(Irr_IAttributes_getAttributeAsRectByIndex(handle,index),True)
	EndMethod
	
	Method setRect3diAttributeByIndex(index:Int,rect:Recti)
		Irr_IAttributes_setRect3diAttributeByIndex(handle,index,rect.handle)
	EndMethod

	Rem
		matrix attribute

	EndRem

	Method addMatrix(attributeName:String,matrix:Matrix4)
		Irr_IAttributes_addMatrix(handle,attributeName,matrix.handle)
	EndMethod
	
	Method setMatrixAttributeByName(attributeName:String,matrix:Matrix4)
		Irr_IAttributes_setMatrixAttributeByName(handle,attributeName,matrix.handle)
	EndMethod
	
	Method getAttributeAsMatrixByName:Matrix4(attributeName:String)
		Return Matrix4.createFromHandle(Irr_IAttributes_getAttributeAsMatrixByName(handle,attributeName),True)
	EndMethod
	
	Method getAttributeAsMatrixByIndex:Matrix4(index:Int)
		Return Matrix4.createFromHandle(Irr_IAttributes_getAttributeAsMatrixByIndex(handle,index),True)
	EndMethod
	
	Method setMatrixAttributeByIndex(index:Int,matrix:Matrix4)
		Irr_IAttributes_setMatrixAttributeByIndex(handle,index,matrix.handle)
	EndMethod

	Rem
		quaternion attribute
	EndRem

	Method addQuaternion(attributeName:String,quat:Quaternion)
		Irr_IAttributes_addQuaternion(handle,attributeName,quat.handle)
	EndMethod
	
	Method setQuaternionAttributeByName(attributeName:String,quat:Quaternion)
		Irr_IAttributes_setQuaternionAttributeByName(handle,attributeName,quat.handle)
	EndMethod
	
	Method getAttributeAsQuaternionByName:Quaternion(attributeName:String)
		Return Quaternion.createFromHandle(Irr_IAttributes_getAttributeAsQuaternionByName(handle,attributeName),True)
	EndMethod
	
	Method getAttributeAsQuaterionByIndex:Quaternion(index:Int)
		Return Quaternion.createFromHandle(Irr_IAttributes_getAttributeAsQuaternionByIndex(handle,index),True)
	EndMethod
	
	Method setQuaternionAttributeByIndex(index:Int,quat:Quaternion)
		Irr_IAttributes_setQuaternionAttributeByIndex(handle,index,quat.handle)
	EndMethod
	
	Rem
		3d bounding box
	EndRem

	Method addBox3d(attributeName:String,box:AABbox3df)
		Irr_IAttributes_addBox3d(handle,attributeName,box.handle)
	EndMethod
	
	Method setAABbox3dfAttributeByName(attributeName:String,box:AABbox3df)
		Irr_IAttributes_setAABbox3dfAttributeByName(handle,attributeName,box.handle)
	EndMethod
	
	Method getAttributeAsBox3dByName:AABbox3df(attributeName:String)
		Return AABbox3df.createFromHandle(Irr_IAttributes_getAttributeAsBox3dByName(handle,attributeName),True)
	EndMethod
	
	Method getAttributeAsBox3dByIndex:AABbox3df(index:Int)
		Return AABbox3df.createFromHandle(Irr_IAttributes_getAttributeAsBox3dByIndex(handle,index),True)
	EndMethod
	
	Method setAABbox3dfAttributeByIndex(index:Int,box:AABbox3df)
		Irr_IAttributes_setAABbox3dfAttributeByIndex(handle,index,box.handle)
	EndMethod
	
	Rem
		plane
	EndRem
	
	Method addPlane3d(attributeName:String,plane:Plane3df)
		Irr_IAttributes_addPlane3d(handle,attributeName,plane.handle)
	EndMethod
	
	Method setPlane3dfAttributeByName(attributeName:String,plane:Plane3df)
		Irr_IAttributes_setPlane3dfAttributeByName(handle,attributeName,plane.handle)
	EndMethod
	
	Method getAttributeAsPlane3dByName:Plane3df(attributeName:String)
		Return Plane3df.createFromHandle(Irr_IAttributes_getAttributeAsPlane3dByName(handle,attributeName),True)
	EndMethod
	
	Method getAttributeAsPlane3dByIndex:Plane3df(index:Int)
		Return Plane3df.createFromHandle(Irr_IAttributes_getAttributeAsPlane3dByIndex(handle,index),True)
	EndMethod
	
	Method setPlane3dfAttributeByIndex(index:Int,plane:Plane3df)
		Irr_IAttributes_setPlane3dfAttributeByIndex(handle,index,plane.handle)
	EndMethod

	Rem
		3d triangle
	EndRem

	Method addTriangle3d(attributeName:String,tri:Triangle3df)
		Irr_IAttributes_addTriangle3d(handle,attributeName,tri.handle)
	EndMethod
	
	Method setTriangle3dfAttributeByName(attributeName:String,tri:Triangle3df)
		Irr_IAttributes_setTriangle3dfAttributeByName(handle,attributeName,tri.handle)
	EndMethod
	
	Method getAttributeAsTriangle3dByName:Triangle3df(attributeName:String)
		Return Triangle3df.createFromHandle(Irr_IAttributes_getAttributeAsTriangle3dByName(handle,attributeName),True)
	EndMethod
	
	Method getAttributeAsTriangle3dByIndex:Triangle3df(index:Int)
		Return Triangle3df.createFromHandle(Irr_IAttributes_getAttributeAsTriangle3dByIndex(handle,index),True)
	EndMethod
	
	Method setTriangle3dfAttributeByIndex(index:Int,tri:Triangle3df)
		Irr_IAttributes_setTriangle3dfAttributeByIndex(handle,index,tri.handle)
	EndMethod

	Rem
		line 2d
	EndRem

	Method addLine2d(attributeName:String,line:Line2df)
		Irr_IAttributes_addLine2d(handle,attributeName,line.handle)
	EndMethod
	
	Method setLine2dfAttributeByName(attributeName:String,line:Line2df)
		Irr_IAttributes_setLine2dfAttributeByName(handle,attributeName,line.handle)
	EndMethod
	
	Method getAttributeAsLine2dByName:Line2df(attributeName:String)
		Return Line2df.createFromHandle(Irr_IAttributes_getAttributeAsLine2dByName(handle,attributeName),True)
	EndMethod

	Method getAttributeAsLine2dByIndex:Line2df(index:Int)
		Return Line2df.createFromHandle(Irr_IAttributes_getAttributeAsLine2dByIndex(handle,index),True)
	EndMethod
	
	Method setLine2dfAttributeByIndex(index:Int,line:Line2df)
		Irr_IAttributes_setLine2dfAttributeByIndex(handle,index,line.handle)
	EndMethod

	Rem
		line 3d
	EndRem

	Method addLine3d(attributeName:String,line:Line3df)
		Irr_IAttributes_addLine3d(handle,attributeName,line.handle) 
	EndMethod
	
	Method setLine3dfAttributeByName(attributeName:String,line:Line3df)
		Irr_IAttributes_setLine3dfAttributeByName(handle,attributeName,line.handle)
	EndMethod
	
	Method getAttributeAsLine3dByName:Line3df(attributeName:String)
		Return Line3df.createFromHandle(Irr_IAttributes_getAttributeAsLine3dByName(handle,attributeName),True)
	EndMethod
	
	Method getAttributeAsLine3dByIndex:Line3df(index:Int)
		Return Line3df.createFromHandle(Irr_IAttributes_getAttributeAsLine3dByIndex(handle,index),True)
	EnDMethod
	
	Method setLine3dfAttributeByIndex(index:Int,line:Line3df)
		Irr_IAttributes_setLin3dfAttributeByIndex(handle,index,line.handle)
	EndMethod

	Rem
		user pointer
	EndRem
	
	Method adduserPointer(attributeName:String,userPointer:Byte Ptr)
		Irr_IAttributes_addUserPointer(handle,attributeName,userPointer)
	EndMethod
	
	Method setUserPointerAttributeByName(attributeName:String,userPointer:Byte Ptr)
		Irr_IAttributes_setUserPointerAttributeByName(handle,attributeName,userPointer)
	EndMethod
	
	Method getAttributeAsUserPointerByName:Byte Ptr(attributeName:String)
		Return Irr_IAttributes_getAttributeAsUserPointerByName(handle,attributeName)
	EndMethod
	
	Method getAttributeAsUserPointerByIndex:Byte Ptr(index:Int)
		Return Irr_IAttributes_getAttributeAsUserPointerByIndex(handle,index)
	EndMethod
	
	Method setUserPointerAttributeByIndex(index:Int,userPointer:Byte Ptr)
		Irr_IAttributes_setUserPointerAttributeByIndex(handle,index,userPointer)
	EndMethod
	
	'! Reads attributes from a xml file.
	'! \param readCurrentElementOnly: If set to true, reading only works if current element has the name 'attributes'.
	'! IF set to false, the first appearing list attributes are read.
	Method read:Int(reader:IXMLReader, bReadCurrentElementOnly:Int=False, elementName:String = Null)
		Return Irr_IAttributes_read(handle,reader.handle,bReadCurrentElementOnly,StringToWChar(elementName))
	EndMethod
	
	'! Write these attributes into a xml file
	Method write:Int(writer:IXMLWriter, bWriteXMLHeader:Int = False, elementName:String = Null)
		Return Irr_IAttributes_write(handle,writer.handle,bWriteXMLHeader,StringToWChar(elementName))
	EndMethod
	
	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return Irr_IAttributes_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return Irr_IAttributes_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		Irr_IAttributes_grab(handle)
	EndMethod
	
	Method Delete()
		If (mustdelete) Then drop()
	EndMethod

EndType



