' ------------------------------------------------------------------------------------
' irr::core::ICollisionCallback wrappers
' ------------------------------------------------------------------------------------

Public

Type ICollisionCallback Extends IReferenceCounted

	Method OnCollision(animator:ISceneNodeAnimatorCollisionResponse, newPosition:Vector3df)
	EndMethod

	Function generate:ICollisionCallback()
		Return New ICollisionCallback
	EndFunction

	Method bind(rec:Int) Final
		Super.bind(rec)
		IrrScene_BMXICollisionCallback_bindOnCollision( ..
			translate(rec).handle, ..
			OnCollisionBinding)
	EndMethod

	Function OnCollisionBinding(binding:Int, animator:Int, newPosition:Int) Final
		' get the object whose node is firing
		Local tempobj:ICollisionCallback = ICollisionCallback(translate(binding))
		
		If tempobj<>Null
	  		' get the node
  			Local tempanim:ISceneNodeAnimatorCollisionResponse = ISceneNodeAnimatorCollisionResponse.createFromHandle(animator, False)

  			' fire the event
	  		tempobj.OnCollision(tempanim, Vector3df.createFromHandle(newPosition, False))
		Else
			Throw "Unable to find bound object"
		EndIf
	EndFunction
		
	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrScene_ICollisionCallback_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrScene_ICollisionCallback_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrScene_ICollisionCallback_grab(handle)
	EndMethod

	Method Delete()
		If (mustdelete) Then drop()
	EndMethod
	
	Method getBinderHandle:Int()
		Return IrrScene_BMXICollisionCallback_getBinder(handle)
	EndMethod
		
	'--------------------------------------------
	' create/delete routines
	'--------------------------------------------
		
	Function create:ICollisionCallback(pGenFunc:Byte Ptr) Final
		Return createFromHandle(pGenFunc,IrrScene_BMXICollisionCallback_new())
	EndFunction
	
	Function createFromHandle:ICollisionCallback(pGenFunc:Byte Ptr,pHandle:Int,bMustDel:Int=True) Final
		If Not pHandle Then Return Null
		' use the funtion pointer to generate
		Local generate:ICollisionCallback()=pGenFunc
		
		Local oRetVal:ICollisionCallback=generate()

		' turn binding on if this is the first
		oRetVal.bIsBound=bMustDel

		' since this is the first one, this is always mustdel
		oRetVal.attach(pHandle,bMustDel)
		
		Return oRetVal
	EndFunction
	
	Method Destroy()
		IrrScene_BMXICollisionCallback_del(handle)
	EndMethod	
EndType

