' ------------------------------------------------------------------------------------
'  irr::scene::IMeshBuffer wrappers
' ------------------------------------------------------------------------------------
Public

Type IMeshBuffer Extends IReferenceCounted

	Function createFromHandle:IMeshBuffer(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:IMeshBuffer=New IMeshBuffer
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction
	
	Method getBoundingBox:AABbox3df()
		Return AABbox3df.createFromHandle(IrrScene_IMeshBuffer_getBoundingBox(handle),True)
	EndMethod
	
	Method getIndexCount:Int()
		Return IrrScene_IMeshBuffer_getIndexCount(handle)
	EndMethod
	
	Method getIndices:Short[]()
	
		Local indices_ptr:Short Ptr = IrrScene_IMeshBuffer_getIndices(handle)
		Local size:Int = getIndexCount()
		Local indices:Short[]
		If size > 0 Then
			indices = New Short[size]
			For Local i:Int=0 Until size
				indices[i] = indices_ptr[i]
			Next
		EndIf
		
		Return indices
	EndMethod
	
	Method getMaterial:SMaterial()
		' do not delete
		Return SMaterial.createFromHandle(IrrScene_IMeshBuffer_getMaterial(handle),False)
	EndMethod
	
	Method getVertexCount:Int()
		Return IrrScene_IMeshBuffer_getVertexCount(handle)
	EndMethod
	
	Method getVertexType:Int()
		Return IrrScene_IMeshBuffer_getVertexType(handle)
	EndMethod

	Method getVertices:S3DVertex[]()
		Local vert_type:Int = getVertexType()

		Local ptrs:Int Ptr	
		Local objs:S3DVertex[] = Null				

		Local size:Int = IrrScene_IMeshBuffer_getVertices(handle,Varptr(ptrs))

		If size>0 Then		
			objs=New S3DVertex[size]

			For Local i:Int=0 Until size
				Select vert_type
					Case EVT_STANDARD
						objs[i]=S3DVertexDefault.createFromHandle(ptrs[i],False)
					Case EVT_2TCOORDS
						objs[i]=S3DVertex2TCoords.createFromHandle(ptrs[i],False)
					Case EVT_TANGENTS
						objs[i]=S3DVertexTangents.createFromHandle(ptrs[i],False)					
				End Select
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		Return objs	
	EndMethod
	
	Method fillS3DVertexVerticesPtrs(pos:Float Ptr,normals:Float Ptr,tcoords:Float Ptr,colors:Float Ptr,bSwap:Int)
		IrrScene_IMeshBuffer_fillS3DVertexVerticesPtrs(handle,pos,normals,tcoords,colors,bSwap)
	EndMethod	
	Method fillS3DVertex2TCoordsVerticesPtrs(pos:Float Ptr,normals:Float Ptr,tcoords:Float Ptr,colors:Float Ptr,tcoords2:Float Ptr,bSwap:Int)
		IrrScene_IMeshBuffer_fillS3DVertex2TCoordsVerticesPtrs(handle,pos,normals,tcoords,colors,tcoords2,bSwap)
	EndMethod	
	Method fillS3DVertexTangentsVerticesPtrs(pos:Float Ptr,normals:Float Ptr,tcoords:Float Ptr,colors:Float Ptr,tangents:Float Ptr,binormals:Float Ptr,bSwap:Int)	
		IrrScene_IMeshBuffer_fillS3DVertexTangentsVerticesPtrs(handle,pos,normals,tcoords,colors,tangents,binormals,bSwap)
	EndMethod
	
	Method fillIndicesPtr(indicies:Short Ptr,bSwap:Int)
		IrrScene_IMeshBuffer_fillIndicesPtr(handle,indicies,bSwap)
	EndMethod

	Method getVertex:S3DVertex(idx:Int)
		Local vert_type:Int = getVertexType()
		Local vert:Int = IrrScene_IMeshBuffer_getVertex(handle,idx)
		
		Select vert_type
			Case EVT_STANDARD
				Return S3DVertexDefault.createFromHandle(vert,False)
			Case EVT_2TCOORDS
				Return S3DVertex2TCoords.createFromHandle(vert,False)
			Case EVT_TANGENTS
				Return S3DVertexTangents.createFromHandle(vert,False)					
		End Select
	EndMethod

	Method setBoundingBox(box:AABbox3df)
		IrrScene_IMeshBuffer_setBoundingBox(handle,box.handle)
	EndMethod
	
	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrScene_IMeshBuffer_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrScene_IMeshBuffer_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrScene_IMeshBuffer_grab(handle)
	EndMethod
	
	Method Delete()
		If (mustdelete) Then drop()
	EndMethod

	Method append(vertices:S3DVertex[], indices:Short[])
		Local vertices_size:Int = 0
		Local indices_size:Int = 0
		Local vertices_array:Int[]=GetHandleArray(vertices)
		If vertices_array Then vertices_size=vertices_array.Dimensions()[0]
		If indices Then indices_size = indices.Dimensions()[0]
		
		IrrScene_IMeshBuffer_append(handle, vertices_array,vertices_size, indices, indices_size)
	EndMethod

	Method appendFromOther(other:IMeshBuffer)
		IrrScene_IMeshBuffer_appendFromOther(handle, other.handle)
	EndMethod

	Method recalculateBoundingBox()
		IrrScene_IMeshBuffer_recalculateBoundingBox(handle)
	EndMethod

	Method getIndexType:Int() ' returns E_INDEX_TYPE
		Return IrrScene_IMeshBuffer_getIndexType(handle) 
	EndMethod
	
	Method getPosition:Vector3df(i:Int)
		Return Vector3df.CreateFromHandle(IrrScene_IMeshBuffer_getPosition(handle, i), True)
	EndMethod
	
	Method getNormal:Vector3df(i:Int)
		Return Vector3df.CreateFromHandle(IrrScene_IMeshBuffer_getNormal(handle, i), True)
	EndMethod
	
	Method getTCoords:Vector2df(i:Int)
		Return Vector2df.CreateFromHandle(IrrScene_IMeshBuffer_getTCoords(handle, i), True)
	EndMethod
		
	Method getHardwareMappingHint_Vertex:Int() ' returns E_HARDWARE_MAPPING
		Return IrrScene_IMeshBuffer_getHardwareMappingHint_Vertex(handle) 
	EndMethod
	
	Method getHardwareMappingHint_Index:Int() ' returns E_HARDWARE_MAPPING
		Return IrrScene_IMeshBuffer_getHardwareMappingHint_Index(handle)
	EndMethod
	
	Method setHardwareMappingHint(E_HARDWARE_MAPPING:Int, E_BUFFER_TYPE:Int = EBT_VERTEX_AND_INDEX)
		IrrScene_IMeshBuffer_setHardwareMappingHint(handle, E_HARDWARE_MAPPING, E_BUFFER_TYPE)
	EndMethod
	
	Method setDirty(E_BUFFER_TYPE:Int = EBT_VERTEX_AND_INDEX)
		IrrScene_IMeshBuffer_setDirty(handle, E_BUFFER_TYPE) 
	EndMethod
	
	Method getChangedID_Vertex:Int()
		Return IrrScene_IMeshBuffer_getChangedID_Vertex(handle)
	EndMethod
	
	Method getChangedID_Index:Int()
		Return IrrScene_IMeshBuffer_getChangedID_Index(handle)
	EndMethod

EndType


