' ------------------------------------------------------------------------------------
'  irr::scene::IParticleSystemSceneNode wrappers (parent is ISceneNode)
' ------------------------------------------------------------------------------------
Public

Type IParticleSystemSceneNode Extends ISceneNodeDefault Final

	Function createFromHandle:IParticleSystemSceneNode(pHandle:Int,bMustDel:Int=False,pGenFunc:Byte Ptr=Null)
		If Not pHandle Then Return Null
		Local oRetVal:IParticleSystemSceneNode=New IParticleSystemSceneNode
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Method addAffector(affector:IParticleAffector)			
		IrrScene_IParticleSystemSceneNode_addAffector(handle,affector.handle)
	EndMethod
		
	Method createAttractionAffector:IParticleAttractionAffector(point:Vector3df, speed:Float = 1.0, bAttract:Int = True, bAffectX:Int = True, bAffectY:Int = True, bAffectZ:Int = True)
		Return IParticleAttractionAffector.createFromHandle(IrrScene_IParticleSystemSceneNode_createAttractionAffector(handle, point.handle, speed, bAttract, bAffectX, bAffectY, bAffectZ), True)
	EndMethod
	
	Method createFadeOutParticleAffector:IParticleFadeOutAffector(targetColor:SColor=Null,timeNeededToFadeOut:Int=1000)
		If targetColor=Null Then targetColor=New SColor ' default handle to 0
		Return IParticleFadeOutAffector.createFromHandle(IrrScene_IParticleSystemSceneNode_createFadeOutParticleAffector(handle,targetColor.handle,timeNeededToFadeOut),True)
	EndMethod

	Method createGravityAffector:IParticleGravityAffector(gravity:Vector3df=Null,timeForceLost:Int=1000)
		If gravity=Null Then gravity=New Vector3df ' default handle to 0
		Return IParticleGravityAffector.createFromHandle(IrrScene_IParticleSystemSceneNode_createGravityAffector(handle,gravity.handle,timeForceLost),True)
	EndMethod

	Method createRotationAffector:IParticleRotationAffector(speed:Vector3df = Null, pivotPoint:Vector3df = Null)
		Return IParticleRotationAffector.createFromHandle(IrrScene_IParticleSystemSceneNode_createRotationAffector(handle, _HANDLE(speed), _HANDLE(pivotPoint) ), True)
	EndMethod
	
	Method createBoxEmitter:IParticleEmitter(box:AABbox3df=Null,direction:Vector3df=Null,minParticlesPerSecond:Int=5,maxParticlePerSecond:Int=10,minStartColor:SColor=Null,maxStartColor:SColor=Null,lifeTimeMin:Int=2000,lifeTimeMax:Int=4000,maxAngleDegrees:Int=0, minStartSize:Dimension2df = Null, maxStartSize:Dimension2df = Null)
		If box=Null Then box=New AABbox3df ' default handle to 0
		If direction=Null Then direction=New Vector3df ' default handle to 0
		If minStartColor=Null Then minStartColor=New SColor ' default handle to 0
		If maxStartColor=Null Then maxStartColor=New SColor ' default handle to 0	
		Return IParticleEmitter.createFromHandle(IrrScene_IParticleSystemSceneNode_createBoxEmitter(handle,box.handle,direction.handle,minParticlesPerSecond,maxParticlePerSecond,minStartColor.handle,maxStartColor.handle,lifeTimeMin,lifeTimeMax,maxAngleDegrees, _HANDLE(minStartSize), _HANDLE(maxStartSize)))
	EndMethod
	
	Method createPointEmitter:IParticleEmitter(direction:Vector3df=Null,minParticlesPerSecond:Int=5,maxParticlePerSecond:Int=10,minStartColor:SColor=Null,maxStartColor:SColor=Null,lifeTimeMin:Int=2000,lifeTimeMax:Int=4000,maxAngleDegrees:Int=0, minStartSize:Dimension2df = Null, maxStartSize:Dimension2df = Null)
		If direction=Null Then direction=New Vector3df ' default handle to 0
		If minStartColor=Null Then minStartColor=New SColor ' default handle to 0
		If maxStartColor=Null Then maxStartColor=New SColor ' default handle to 0	
		'  you must call drop For this when done.
		Return IParticleEmitter.createFromHandle(IrrScene_IParticleSystemSceneNode_createPointEmitter(handle,direction.handle,minParticlesPerSecond,maxParticlePerSecond,minStartColor.handle,maxStartColor.handle,lifeTimeMin,lifeTimeMax,maxAngleDegrees, _HANDLE(minStartSize), _HANDLE(maxStartSize)))
	EndMethod

	Method createAnimatedMeshSceneNodeEmitter:IParticleAnimatedMeshSceneNodeEmitter(node:IAnimatedMeshSceneNode, bUseNormalDirection:Int = True, direction:Vector3df = Null, normalDirectionModifier:Float = 100.0, mbNumber:Int = -1, bEveryMeshVertex:Int = False, minParticlesPerSecond:Int = 5, maxParticlesPerSecond:Int = 10,	minStartColor:SColor = Null, maxStartColor:SColor = Null, lifeTimeMin:Int = 2000, lifeTimeMax:Int = 4000, maxAngleDegrees:Int = 0, minStartSize:Dimension2df = Null, maxStartSize:Dimension2df = Null)	
		Return IParticleAnimatedMeshSceneNodeEmitter.createFromHandle(IrrScene_IParticleSystemSceneNode_createAnimatedMeshSceneNodeEmitter(handle, node.handle, bUseNormalDirection, _HANDLE(direction), normalDirectionModifier, mbNumber, bEveryMeshVertex, minParticlesPerSecond, maxParticlesPerSecond, _HANDLE(minStartColor), _HANDLE(maxStartColor), lifeTimeMin, lifeTimeMax, maxAngleDegrees, _HANDLE(minStartSize), _HANDLE(maxStartSize) ), True)
	EndMethod
		
	Method createCylinderEmitter:IParticleCylinderEmitter(center:Vector3df, radius:Float, normal:Vector3df, length:Float, bOutlineOnly:Int = False, direction:Vector3df = Null, minParticlesPerSecond:Int = 5, maxParticlesPerSecond:Int = 10,	minStartColor:SColor = Null, maxStartColor:SColor = Null, lifeTimeMin:Int = 2000, lifeTimeMax:Int = 4000, maxAngleDegrees:Int = 0, minStartSize:Dimension2df = Null, maxStartSize:Dimension2df = Null)
		Return IParticleCylinderEmitter.createFromHandle(IrrScene_IParticleSystemSceneNode_createCylinderEmitter(handle, ..
				center.handle, radius, normal.handle, length, bOutlineOnly, ..
				_HANDLE(direction), minParticlesPerSecond, maxParticlesPerSecond, ..
				_HANDLE(minStartColor), _HANDLE(maxStartColor), lifeTimeMin, lifeTimeMax, maxAngleDegrees, _HANDLE(minStartSize), _HANDLE(maxStartSize)), True)
	EndMethod
		
	Method createMeshEmitter:IParticleMeshEmitter(mesh:IMesh, bUseNormalDirection:Int = True, direction:Vector3df = Null, normalDirectionModifier:Float = 100.0, mbNumber:Int = -1, bEveryMeshVertex:Int = False, minParticlesPerSecond:Int = 5, maxParticlesPerSecond:Int = 10,	minStartColor:SColor = Null, maxStartColor:SColor = Null, lifeTimeMin:Int = 2000, lifeTimeMax:Int = 4000, maxAngleDegrees:Int = 0, minStartSize:Dimension2df = Null, maxStartSize:Dimension2df = Null)
		Return IParticleMeshEmitter.createFromHandle(IrrScene_IParticleSystemSceneNode_createMeshEmitter(handle, mesh.handle, bUseNormalDirection, _HANDLE(direction), normalDirectionModifier, mbNumber, bEveryMeshVertex, minParticlesPerSecond, maxParticlesPerSecond, _HANDLE(minStartColor), _HANDLE(maxStartColor), lifeTimeMin, lifeTimeMax, maxAngleDegrees, _HANDLE(minStartSize), _HANDLE(maxStartSize) ), True)
	EndMethod
		
	Method createRingEmitter:IParticleRingEmitter(center:Vector3df, radius:Float, ringThickness:Float, direction:Vector3df = Null, minParticlesPerSecond:Int = 5, maxParticlesPerSecond:Int = 10,	minStartColor:SColor = Null, maxStartColor:SColor = Null, lifeTimeMin:Int = 2000, lifeTimeMax:Int = 4000, maxAngleDegrees:Int = 0, minStartSize:Dimension2df = Null, maxStartSize:Dimension2df = Null)
		Return IParticleRingEmitter.createFromHandle(IrrScene_IParticleSystemSceneNode_createRingEmitter(handle, center.handle, radius, ringThickness, ..
			_HANDLE(direction), minParticlesPerSecond, maxParticlesPerSecond, ..
			_HANDLE(minStartColor), _HANDLE(maxStartColor), lifeTimeMin, lifeTimeMax, maxAngleDegrees, _HANDLE(minStartSize), _HANDLE(maxStartSize)), True)
	EndMethod
	
	Method createSphereEmitter:IParticleSphereEmitter(center:Vector3df, radius:Float, direction:Vector3df = Null, minParticlesPerSecond:Int = 5, maxParticlesPerSecond:Int = 10,	minStartColor:SColor = Null, maxStartColor:SColor = Null, lifeTimeMin:Int = 2000, lifeTimeMax:Int = 4000, maxAngleDegrees:Int = 0, minStartSize:Dimension2df = Null, maxStartSize:Dimension2df = Null)
		Return IParticleSphereEmitter.createFromHandle(IrrScene_IParticleSystemSceneNode_createSphereEmitter(handle, center.handle, radius, ..
			_HANDLE(direction), minParticlesPerSecond, maxParticlesPerSecond, ..
			_HANDLE(minStartColor), _HANDLE(maxStartColor), lifeTimeMin, lifeTimeMax, maxAngleDegrees, _HANDLE(minStartSize), _HANDLE(maxStartSize)), True)
	EndMethod

	Method removeAllAffectors()
		IrrScene_IParticleSystemSceneNode_removeAllAffectors(handle)
	EndMethod
	
	Method setEmitter(emitter:IParticleEmitter=Null)
		If emitter=Null Then emitter=New IParticleEmitter ' default the handle to 0
		IrrScene_IParticleSystemSceneNode_setEmitter(handle,emitter.handle)
	EndMethod

	Method getEmitter:IParticleEmitter()
		Return IParticleEmitter.createFromHandle(IrrScene_IParticleSystemSceneNode_getEmitter(handle), False)
	EndMethod
	
	Method setParticlesAreGlobal(bGlobal:Int = True)
		IrrScene_IParticleSystemSceneNode_setParticlesAreGlobal(handle,bGlobal)
	EndMethod
	
	Method setParticleSize(size:Dimension2df=Null)
		If size=Null Then size=New Dimension2df ' default handle to 0
		IrrScene_IParticleSystemSceneNode_setParticleSize(handle,size.handle)
	EndMethod

EndType
