' ------------------------------------------------------------------------------------
'  irr::scene::ISceneNode wrappers
' ------------------------------------------------------------------------------------
Public

Type ISceneNode Extends IAttributeExchangingObject

	Method New()
		' setup the cast to ireferencecounted 
		cast_IReferenceCounted_func = IrrScene_ISceneNode_castToIReferenceCounted
	EndMethod

	' --------------------------------------------------------------------------------
	' override these in extended type to implement new functionality
	' --------------------------------------------------------------------------------

	Method getBoundingBox:AABbox3df() Abstract

	Method render() Abstract

	Method getSceneManager:ISceneManager()
		' only available with bound types
		If bIsBound Then Return ISceneManager.createFromHandle(BMXISceneNode_getSceneManager(handle),False) Else Return Null
	EndMethod

	' not overridable in subtypes
	Method getAnimators:ISceneNodeAnimator[]() Final
		Local ptrs:Int Ptr	
		Local objs:ISceneNodeAnimator[]=Null				

		Local size:Int=IrrScene_ISceneNode_getAnimators(handle,Varptr(ptrs))

		If size>0 Then		
			objs=New ISceneNodeAnimator[size]

			For Local i:Int=0 Until size
				objs[i]=ISceneNodeAnimator.createFromHandle(ptrs[i],False)
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		Return objs	
	EndMethod
	
	Method addAnimator(animator:ISceneNodeAnimator)
		If Not bIsBound Then
			IrrScene_ISceneNode_addAnimator(handle,animator.handle)
		Else
			BMXISceneNode_addAnimator(handle,animator.handle)
		EndIf
	EndMethod
	
	Method addChild(child:ISceneNode)
		If Not bIsBound Then
			IrrScene_ISceneNode_addChild(handle,child.handle)
		Else
			BMXISceneNode_addChild(handle,child.handle)
		EndIf
	EndMethod
	
	Method getAbsolutePosition:Vector3df()
		If Not bIsBound Then
			Return Vector3df.createFromHandle(IrrScene_ISceneNode_getAbsolutePosition(handle))
		Else
			Return Vector3df.createFromHandle(BMXISceneNode_getAbsolutePosition(handle))
		EndIf
	EndMethod
	
	Method getAbsoluteTransformation:Matrix4()
		If Not bIsBound Then
			Return Matrix4.createFromHandle(IrrScene_ISceneNode_getAbsoluteTransformation(handle),True)
		Else
			Return Matrix4.createFromHandle(BMXISceneNode_getAbsoluteTransformation(handle),True)
		EndIf
	EndMethod
	
	Method getAutomaticCulling:Int()
		If Not bIsBound Then
			Return IrrScene_ISceneNode_getAutomaticCulling(handle)
		Else
			Return BMXISceneNode_getAutomaticCulling(handle)
		EndIf
	EndMethod
		
	Method getID:Int()
		If Not bIsBound Then
			Return IrrScene_ISceneNode_getID(handle)
		Else
			Return BMXISceneNode_getID(handle)
		EndIf
	EndMethod
	
	Method getMaterial:SMaterial(idx:Int)
		'  do Not Delete this pointer
		If Not bIsBound Then
			Return SMaterial.createFromHandle(IrrScene_ISceneNode_getMaterial(handle,idx),False)
		Else
			Return SMaterial.createFromHandle(BMXISceneNode_getMaterial(handle,idx),False)
		EndIf
	EndMethod
	
	Method getMaterialCount:Int()
		If Not bIsBound Then
			Return IrrScene_ISceneNode_getMaterialCount(handle)
		Else
			Return BMXISceneNode_getMaterialCount(handle)
		EndIf
	EndMethod
	
	Method getName:String()
		If Not bIsBound Then
			Return IrrScene_ISceneNode_getName(handle)
		Else
			Return BMXISceneNode_getName(handle)
		EndIf
	EndMethod
	
	Method getType:Int()
		If Not bIsBound Then
			Return IrrScene_ISceneNode_getType(handle)
		Else
			Return BMXISceneNode_getType(handle)
		EndIf
	EndMethod

	Method getParent:ISceneNode()
		Local node:ISceneNode=Null
		'  do Not Delete this pointer
		If Not bIsBound Then
			node=ISceneNode.createFromHandle(IrrScene_ISceneNode_getParent(handle),False)
		Else
			node=ISceneNode.createFromHandle(BMXISceneNode_getParent(handle),False)
		EndIf
		Return node
	EndMethod
	
	Method getPosition:Vector3df()
		If Not bIsBound Then
			Return Vector3df.createFromHandle(IrrScene_ISceneNode_getPosition(handle))
		Else
			Return Vector3df.createFromHandle(BMXISceneNode_getPosition(handle))
		EndIf
	EndMethod
	
	Method getRelativeTransformation:Matrix4()
		If Not bIsBound Then
			Return Matrix4.createFromHandle(IrrScene_ISceneNode_getRelativeTransformation(handle))
		Else
			Return Matrix4.createFromHandle(BMXISceneNode_getRelativeTransformation(handle))
		EndIf
	EndMethod
	
	Method GetRotation:Vector3df()
		If Not bIsBound Then
			Return Vector3df.createFromHandle(IrrScene_ISceneNode_getRotation(handle))
		Else
			Return Vector3df.createFromHandle(BMXISceneNode_getRotation(handle))
		EndIf
	EndMethod
	
	Method GetScale:Vector3df()
		If Not bIsBound Then
			Return Vector3df.createFromHandle(IrrScene_ISceneNode_getScale(handle))
		Else
			Return Vector3df.createFromHandle(BMXISceneNode_getScale(handle))
		EndIf
	EndMethod
	
	Method getTransformedBoundingBox:AABbox3df()
		If Not bIsBound Then
			Return AABbox3df.createFromhandle(IrrScene_ISceneNode_getTransformedBoundingBox(handle)) 
		Else
			Return AABbox3df.createFromhandle(BMXISceneNode_getTransformedBoundingBox(handle))
		EndIf
	EndMethod
	
	Method getTriangleSelector:ITriangleSelector()
		' do not delete
		If Not bIsBound Then
			Return ITriangleSelector.createFromHandle(IrrScene_ISceneNode_getTriangleSelector(handle),False)
		Else
			Return ITriangleSelector.createFromHandle(BMXISceneNode_getTriangleSelector(handle),False)
		EndIf
	EndMethod

	Method isDebugDataVisible:Int()
		If Not bIsBound Then
			Return IrrScene_ISceneNode_isDebugDataVisible(handle)
		Else
			Return BMXISceneNode_isDebugDataVisible(handle)
		EndIf
	EndMethod
	
	Method isVisible:Int()
		If Not bIsBound Then
			Return IrrScene_ISceneNode_isVisible(handle)
		Else
			Return BMXISceneNode_isVisible(handle)
		EndIf
	EndMethod
	
	Method remove()
		If Not bIsBound Then
			IrrScene_ISceneNode_remove(handle)
		Else
			BMXISceneNode_remove(handle)
		EndIf
	EndMethod
	
	Method removeAll()
		If Not bIsBound Then
			IrrScene_ISceneNode_removeAll(handle)
		Else
			BMXISceneNode_removeAll(handle)
		EndIf
	EndMethod
	
	Method removeAnimator(animator:ISceneNodeAnimator)
		If Not bIsBound Then
			IrrScene_ISceneNode_removeAnimator(handle,animator.handle)
		Else
			BMXISceneNode_removeAnimator(handle,animator.handle)
		EndIf
	EndMethod
	
	Method removeAnimators()
		If Not bIsBound Then
			IrrScene_ISceneNode_removeAnimators(handle)
		Else
			BMXISceneNode_removeAnimators(handle)
		EndIf
	EndMethod
	
	Method removeChild(child:ISceneNode)
		If Not bIsBound Then
			IrrScene_ISceneNode_removeChild(handle,child.handle)
		Else
			BMXISceneNode_removeChild(handle,child.handle)
		EndIf
	EndMethod
		
	Method OnRegisterSceneNode()
		If Not bIsBound Then 
			IrrScene_ISceneNode_OnRegisterSceneNode(handle)
		Else
			BMXISceneNode_OnRegisterSceneNode(handle)		
		EndIf
	EndMethod

	Method OnAnimate(timeMs:Int)
		If Not bIsBound Then 
			IrrScene_ISceneNode_OnAnimate(handle,timeMs)
		Else
			BMXISceneNode_OnAnimate(handle,timeMs)
		EndIf
	EndMethod

	Method setAutomaticCulling(E_CULLING_TYPE:Int)
		If Not bIsBound Then
			IrrScene_ISceneNode_setAutomaticCulling(handle,E_CULLING_TYPE)
		Else
			BMXISceneNode_setAutomaticCulling(handle,E_CULLING_TYPE)
		EndIf
	EndMethod
	
	Method setDebugDataVisible(state:Int)
		If Not bIsBound Then
			IrrScene_ISceneNode_setDebugDataVisible(handle, state)
		Else
			BMXISceneNode_setDebugDataVisible(handle, state)
		EndIf
	EndMethod
	
	Method setID(id:Int)
		If Not bIsBound Then
			IrrScene_ISceneNode_setID(handle,id)
		Else
			BMXISceneNode_setID(handle,id)
		EndIf
	EndMethod
	
	Method setMaterialFlag(E_MATERIAL_FLAG_flag:Int,bNewvalue:Int)
		If Not bIsBound Then
			IrrScene_ISceneNode_setMaterialFlag(handle,E_MATERIAL_FLAG_flag,bNewvalue)
		Else
			BMXISceneNode_setMaterialFlag(handle,E_MATERIAL_FLAG_flag,bNewvalue)
		EndIf
	EndMethod
	
	Method setMaterialTexture(textureLayer:Int,texture:ITexture)
		If Not bIsBound Then
			IrrScene_ISceneNode_setMaterialTexture(handle,textureLayer,texture.handle)
		Else
			BMXISceneNode_setMaterialTexture(handle,textureLayer,texture.handle)
		EndIf
	EndMethod
	
	Method setMaterialType(E_MATERIAL_TYPE_newType:Int)
		If Not bIsBound Then
			IrrScene_ISceneNode_setMaterialType(handle,E_MATERIAL_TYPE_newType)
		Else
			BMXISceneNode_setMaterialType(handle,E_MATERIAL_TYPE_newType)
		EndIf
	EndMethod
	
	Method setMaterial(idx:Int,newmat:SMaterial)
		If Not bIsBound Then
			IrrScene_ISceneNode_setMaterial(handle,idx,newmat.handle)
		Else
			BMXISceneNode_setMaterial(handle,idx,newmat.handle)
		EndIf
	EndMethod

	Method setName(name:String)
		If Not bIsBound Then
			IrrScene_ISceneNode_setName(handle,name)
		Else
			BMXISceneNode_setName(handle,name)
		EndIf
	EndMethod
	
	Method setParent(newParent:ISceneNode=Null)
		If newParent=Null Then newParent=New ISceneNodeDefault ' default handle to 0
		If Not bIsBound Then
			IrrScene_ISceneNode_setParent(handle,newParent.handle)
		Else
			BMXISceneNode_setParent(handle,newParent.handle)
		EndIf
	EndMethod
	
	Method setPosition(newpos:Vector3df)
		If Not bIsBound Then
			IrrScene_ISceneNode_setPosition(handle,newpos.handle)
		Else
			BMXISceneNode_setPosition(handle,newpos.handle)
		EndIf
	EndMethod
	
	Method SetRotation(rotation:Vector3df)
		If Not bIsBound Then
			IrrScene_ISceneNode_setRotation(handle,rotation.handle)
		Else
			BMXISceneNode_setRotation(handle,rotation.handle)
		EndIf
	EndMethod
	
	Method SetScale(scale:Vector3df)
		If Not bIsBound Then
			IrrScene_ISceneNode_setScale(handle,scale.handle)
		Else
			BMXISceneNode_setScale(handle,scale.handle)
		EndIf
	EndMethod
	
	Method setTriangleSelector(selector:ITriangleSelector)
		If Not bIsBound Then
			IrrScene_ISceneNode_setTriangleSelector(handle,selector.handle)
		Else
			BMXISceneNode_setTriangleSelector(handle,selector.handle)
		EndIf
	EndMethod
	
	Method setVisible(bIsVisible:Int)
		If Not bIsBound Then
			IrrScene_ISceneNode_setVisible(handle,bIsVisible)
		Else
			BMXISceneNode_setVisible(handle,bIsVisible)
		EndIf
	EndMethod
	
	Method updateAbsolutePosition()
		If Not bIsBound Then
			IrrScene_ISceneNode_updateAbsolutePosition(handle)
		Else
			BMXISceneNode_updateAbsolutePosition(handle)
		EndIf
	EndMethod

	' not overridable in subtypes	
	Method getChildren:ISceneNode[]() Final
		Local ptrs:Int Ptr	
		Local objs:ISceneNode[]=Null				

		Local size:Int=IrrScene_ISceneNode_getChildren(handle,Varptr(ptrs))

		If size>0 Then		
			objs=New ISceneNode[size]

			For Local i:Int=0 Until size
				objs[i]=ISceneNode.createFromHandle(ptrs[i],False)
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		Return objs
	EndMethod

	Method getChild:ISceneNode(idx:Int)
		Return getChildren()[idx]
	EndMethod

	Method setIsDebugObject(bDebugObject:Int)	
		If Not bIsBound Then
			IrrScene_ISceneNode_setIsDebugObject(handle,bDebugObject)
		Else
			BMXISceneNode_setIsDebugObject(handle,bDebugObject)			
		EndIf
	EndMethod
	
	Method isDebugObject:Int()
		If Not bIsBound Then
			Return IrrScene_ISceneNode_isDebugObject(handle)
		Else
			Return BMXISceneNode_isDebugObject(handle)			
		EndIf
	EndMethod
	
	Method serializeAttributes(out:IAttributes,options:SAttributeReadWriteOptions=Null)
		If Not bIsBound Then
			Super.serializeAttributes(out,options)
		Else
			If Not options Then options=New SAttributeReadWriteOptions ' default handle to 0
			BMXISceneNode_serializeAttributes(handle,out.handle,options.handle)
		EndIf
	EndMethod

	Method deserializeAttributes(in:IAttributes,options:SAttributeReadWriteOptions=Null)
		If Not bIsBound Then
			Super.deserializeAttributes(in,options)
		Else
			If Not options Then options=New SAttributeReadWriteOptions ' default handle to 0
			BMXISceneNode_deserializeAttributes(handle,in.handle,options.handle)	
		EndIf
	EndMethod

	Method clone:ISceneNode(newParent:ISceneNode = Null, newManager:ISceneManager = Null)
		If Not bIsBound Then
			Return ISceneNode.createFromHandle(IrrScene_ISceneNode_clone(handle,_HANDLE(newParent),_HANDLE(newManager)),True)
		Else
			Return ISceneNode.createFromHandle(BMXISceneNode_clone(handle,_HANDLE(newParent),_HANDLE(newManager)),True)
		EndIf
	EndMethod
	
	Method Delete()
		If (mustdelete) Then drop()
	EndMethod	

	' TODO: impelement any new node types
	Method cast:ISceneNode(bTakeOwnership:Int=True)
		Local node_cast:ISceneNode=Null
		Local node_type:Int=getType()
		Local bMustDelete:Int=False

		If bTakeOwnership Then 
			bMustDelete=Self.mustdelete
			Self.mustdelete=False
		EndIf
		
		Select node_type

			Rem
			Case ESNT_CUBE			
			' Water Surface Scene Node
			Case ESNT_WATER_SURFACE
			' Sky Box Scene Node
			Case ESNT_SKY_BOX
			EndRem

			' OctTree Scene Node
			Case ESNT_OCT_TREE
				node_cast=IMeshSceneNode.createFromHandle(Self.handle,bMustDelete)

			Case ESNT_SPHERE
				node_cast=IMeshSceneNode.createFromHandle(Self.handle,bMustDelete)
			
			Case ESNT_TEXT
				node_cast=ITextSceneNode.createFromHandle(Self.handle,bMustDelete)
						
			' Terrain Scene Node
			Case ESNT_TERRAIN
				node_cast=ITerrainSceneNode.createFromHandle(Self.handle,bMustDelete)
						
			' Shadow Volume Scene Node
			Case ESNT_SHADOW_VOLUME
				node_cast=IShadowVolumeSceneNode.createFromHandle(Self.handle,bMustDelete)
						
			' Mesh Scene Node
			Case ESNT_MESH
				node_cast=IMeshSceneNode.createFromHandle(Self.handle,bMustDelete)
			
			' Light Scene Node
			Case ESNT_LIGHT
				node_cast=ILightSceneNode.createFromHandle(Self.handle,bMustDelete)
						
			' Dummy Transformation Scene Node
			Case ESNT_DUMMY_TRANSFORMATION
				node_cast=IDummyTransformationSceneNode.createFromHandle(Self.handle,bMustDelete)
			
			' Camera Scene Node
			Case ESNT_CAMERA
				node_cast=ICameraSceneNode.createFromHandle(Self.handle,bMustDelete)
			
			' Maya Camera Scene Node
			Case ESNT_CAMERA_MAYA
				node_cast=ICameraSceneNode.createFromHandle(Self.handle,bMustDelete)
			
			' First Person Shooter style Camera
			Case ESNT_CAMERA_FPS
				node_cast=ICameraSceneNode.createFromHandle(Self.handle,bMustDelete)
			
			' Billboard Scene Node
			Case ESNT_BILLBOARD
				node_cast=IBillboardSceneNode.createFromHandle(Self.handle,bMustDelete)
			
			' Animated Mesh Scene Node
			Case ESNT_ANIMATED_MESH
				node_cast=IAnimatedMeshSceneNode.createFromHandle(Self.handle,bMustDelete)
			
			' Particle System Scene Node
			Case ESNT_PARTICLE_SYSTEM
				node_cast=IParticleSystemSceneNode.createFromHandle(Self.handle,bMustDelete)

			Default
				node_cast=Self
				Self.mustdelete=bMustDelete
		EndSelect

		Return node_cast
	EndMethod

	' --------------------------------------------------------------------------------
	' creation routines
	' --------------------------------------------------------------------------------

	' this should only be called on an extended type
	Function create:ISceneNode(pGenFunc:Byte Ptr,parent:ISceneNode,mgr:ISceneManager,id:Int=-1,pos:Vector3df=Null,rot:Vector3df=Null,scale:Vector3df=Null)
		If parent=Null Then parent=mgr.getRootSceneNode() ' default handle to 0
		If pos=Null Then pos=New Vector3df ' default handle to 0
		If rot=Null Then rot=New Vector3df ' default handle to 0
		If scale=Null Then scale=New Vector3df ' default handle to 0
		Return createFromHandle(BMXISceneNode_new(parent.handle,mgr.handle,id,pos.handle,rot.handle,scale.handle),True,pGenFunc)
	EndFunction

	Function createFromHandle:ISceneNode(pHandle:Int,bMustDel:Int,pGenFunc:Byte Ptr=Null)
		If Not pHandle Then Return Null

		Local oRetVal:ISceneNode=Null
		
		If pGenFunc
			' use the funtion pointer to generate
			Local generate:ISceneNode()=pGenFunc		
			oRetVal=generate()
		Else
			oRetVal=New ISceneNodeDefault
		EndIf

		' turn binding on if this is the first and there was a gen func passed in
		oRetVal.bIsBound=bMustDel And pGenFunc
		
		oRetVal.attach(pHandle,bMustDel)
		
		Return oRetVal.cast()
	EndFunction

	' --------------------------------------------
	' bindings
	' --------------------------------------------
	
	Method bind(rec:Int) Final
		Super.bind(rec)

		BMXISceneNode_bindIsVisible( ..
			translate(rec).handle, ..
			boundIsVisible)			
		BMXISceneNode_bindRender( ..
			translate(rec).handle, ..
			boundRender)
	   	BMXISceneNode_bindOnRegisterSceneNode( ..
			translate(rec).handle, ..
			boundOnRegisterSceneNode)			
	 	BMXISceneNode_bindOnAnimate( ..
			translate(rec).handle, ..
			boundOnAnimate)
		BMXISceneNode_bindAddAnimator( ..
			translate(rec).handle, ..
			boundAddAnimator)			
		BMXISceneNode_bindAddChild( ..
			translate(rec).handle, ..
			boundAddChild)				
		BMXISceneNode_bindGetAbsolutePosition( ..
			translate(rec).handle, ..
			boundGetAbsolutePosition)
		BMXISceneNode_bindGetAbsoluteTransformation( ..
			translate(rec).handle, ..
			boundGetAbsoluteTransformation)
		BMXISceneNode_bindGetAutomaticCulling( ..
			translate(rec).handle, ..
			boundGetAutomaticCulling)				
		BMXISceneNode_bindGetBoundingBox( ..
			translate(rec).handle, ..
			boundGetBoundingBox)
		BMXISceneNode_bindGetID( ..
			translate(rec).handle, ..
			boundGetID)
		BMXISceneNode_bindGetMaterial( ..
			translate(rec).handle, ..
			boundGetMaterial)		
		BMXISceneNode_bindGetMaterialCount( ..
			translate(rec).handle, ..
			boundGetMaterialCount)
		BMXISceneNode_bindGetName( ..
			translate(rec).handle, ..
			boundGetName)
		BMXISceneNode_bindGetType( ..
			translate(rec).handle, ..
			boundGetType)
		BMXISceneNode_bindGetParent( ..
			translate(rec).handle, ..
			boundGetParent)
		BMXISceneNode_bindGetPosition( ..
			translate(rec).handle, ..
			boundGetPosition)	
		BMXISceneNode_bindGetRelativeTransformation( ..
			translate(rec).handle, ..
			boundGetRelativeTransformation)				
		BMXISceneNode_bindGetRotation( ..
			translate(rec).handle, ..
			boundGetRotation)		
		BMXISceneNode_bindGetScale( ..
			translate(rec).handle, ..
			boundGetScale)		
		BMXISceneNode_bindGetTransformedBoundingBox( ..
			translate(rec).handle, ..
			boundGetTransformedBoundingBox)		
		BMXISceneNode_bindGetTriangleSelector( ..
			translate(rec).handle, ..
			boundGetTriangleSelector)
		BMXISceneNode_bindIsDebugDataVisible( ..
			translate(rec).handle, ..
			boundIsDebugDataVisible)
		BMXISceneNode_bindRemove( ..
			translate(rec).handle, ..
			boundRemove)
		BMXISceneNode_bindRemoveAll( ..
			translate(rec).handle, ..
			boundRemoveAll)
		BMXISceneNode_bindRemoveAnimator( ..
			translate(rec).handle, ..
			boundRemoveAnimator)		
		BMXISceneNode_bindRemoveAnimators( ..
			translate(rec).handle, ..
			boundRemoveAnimators)		
		BMXISceneNode_bindRemoveChild( ..
			translate(rec).handle, ..
			boundRemoveChild)		
		BMXISceneNode_bindSetAutomaticCulling( ..
			translate(rec).handle, ..
			boundSetAutomaticCulling)		
		BMXISceneNode_bindSetDebugDataVisible( ..
			translate(rec).handle, ..
			boundSetDebugDataVisible)
		BMXISceneNode_bindSetID( ..
			translate(rec).handle, ..
			boundSetID)
		BMXISceneNode_bindSetMaterialFlag( ..
			translate(rec).handle, ..
			boundSetMaterialFlag)
		BMXISceneNode_bindSetMaterialTexture( ..
			translate(rec).handle, ..
			boundSetMaterialTexture)		
		BMXISceneNode_bindSetMaterialType( ..
			translate(rec).handle, ..
			boundSetMaterialType)		
		BMXISceneNode_bindSetMaterial( ..
			translate(rec).handle, ..
			boundSetMaterial)		
		BMXISceneNode_bindSetName( ..	
			translate(rec).handle, ..
			boundSetName)				
		BMXISceneNode_bindSetParent( ..
			translate(rec).handle, ..
			boundSetParent)
		BMXISceneNode_bindSetPosition( ..
			translate(rec).handle, ..
			boundSetPosition)
		BMXISceneNode_bindSetRotation( ..
			translate(rec).handle, ..
			boundSetRotation)
		BMXISceneNode_bindSetScale( ..
			translate(rec).handle, ..
			boundSetScale)
		BMXISceneNode_bindSetTriangleSelector( ..
			translate(rec).handle, ..
			boundSetTriangleSelector)
		BMXISceneNode_bindSetVisible( ..
			translate(rec).handle, ..
			boundSetVisible)
		BMXISceneNode_bindUpdateAbsolutePosition( ..
			translate(rec).handle, ..
			boundUpdateAbsolutePosition)
		BMXISceneNode_bindSetIsDebugObject( ..
			translate(rec).handle, ..
			boundSetIsDebugObject)		
		BMXISceneNode_bindIsDebugObject( ..
			translate(rec).handle, ..
			boundIsDebugObject)		
		BMXISceneNode_bindSerializeAttributes( ..
			translate(rec).handle, ..
			boundSerializeAttributes)		
		BMXISceneNode_bindDeserializeAttributes( ..
			translate(rec).handle, ..
			boundDeserializeAttributes)
		BMXISceneNode_bindClone( ..
			translate(rec).handle, ..
			boundClone)

	EndMethod
	
	Function boundUpdateAbsolutePosition(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.updateAbsolutePosition()
		Else
			Throw "Unable to find bound object"
		EndIf						
	EndFunction

	Function boundSetVisible(binding:Int,bIsVisible:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setVisible(bIsVisible)
		Else
			Throw "Unable to find bound object"
		EndIf				
	EndFunction

	Function boundSetTriangleSelector(binding:Int,selector:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setTriangleSelector(ITriangleSelector.createFromHandle(selector,False))
		Else
			Throw "Unable to find bound object"
		EndIf			
	EndFunction

	Function boundSetScale(binding:Int,newScale:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setScale(Vector3df.createFromHandle(newScale,False))
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundSetRotation(binding:Int,newRot:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setRotation(Vector3df.createFromHandle(newRot,False))
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundSetPosition(binding:Int,newpos:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setPosition(Vector3df.createFromHandle(newPos,False))
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundSetParent(binding:Int,newParent:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setParent(ISceneNode.createFromHandle(newParent,False))
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundSetName(binding:Int,name:Byte Ptr)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setName(String.FromCString(name))
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundSetMaterial(binding:Int,i:Int,newmat:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setMaterial(i,SMaterial.createFromHandle(newmat,False))
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundSetMaterialType(binding:Int,newType:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setMaterialType(newType)
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundSetMaterialTexture(binding:Int,textureLayer:Int,texture:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setMaterialTexture(textureLayer,ITexture.createFromHandle(texture,False))
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundSetMaterialFlag(binding:Int,flag:Int,bNewvalue:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setMaterialFlag(flag,bNewValue)
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundSetID(binding:Int,id:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setID(id)
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundSetDebugDataVisible(binding:Int,bVisible:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setDebugDataVisible(bVisible)
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundSetAutomaticCulling(binding:Int,bEnabled:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setAutomaticCulling(bEnabled)
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundRemoveChild(binding:Int,child:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.removeChild(ISceneNode.createFromHandle(child,False))
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundRemoveAnimators(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.removeAnimators()
		Else
			Throw "Unable to find bound object"
		EndIf						
	EndFunction

	Function boundRemoveAnimator(binding:Int,animator:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.removeAnimator(ISceneNodeAnimator.createFromHandle(animator,False))
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundRemoveAll(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.removeAll()
		Else
			Throw "Unable to find bound object"
		EndIf						
	EndFunction

	Function boundRemove(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.remove()
		Else
			Throw "Unable to find bound object"
		EndIf						
	EndFunction
	
	Function boundIsDebugDataVisible:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.isDebugDataVisible()
		Else
			Throw "Unable to find bound object"
		EndIf						
	EndFunction

	Function boundGetTriangleSelector:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getTriangleSelector().handle
		Else
			Throw "Unable to find bound object"
		EndIf						
	EndFunction
	
	Function boundGetTransformedBoundingBox:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getTransformedBoundingBox().handle
		Else
			Throw "Unable to find bound object"
		EndIf						
	EndFunction

	Function boundGetScale:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getScale().handle
		Else
			Throw "Unable to find bound object"
		EndIf						
	EndFunction
	
	Function boundGetRotation:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getRotation().handle
		Else
			Throw "Unable to find bound object"
		EndIf					
	EndFunction

	Function boundGetRelativeTransformation:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getRelativeTransformation().handle
		Else
			Throw "Unable to find bound object"
		EndIf					
	EndFunction

	Function boundGetPosition:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getPosition().handle
		Else
			Throw "Unable to find bound object"
		EndIf					
	EndFunction

	Function boundGetParent:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getParent().handle
		Else
			Throw "Unable to find bound object"
		EndIf				
	EndFunction

	Function boundGetName:Byte Ptr(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getName().ToCString()
		Else
			Throw "Unable to find bound object"
		EndIf				
	EndFunction
	
	Function boundGetType:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getType()
		Else
			Throw "Unable to find bound object"
		EndIf				
	EndFunction

	Function boundGetMaterialCount:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getMaterialCount()
		Else
			Throw "Unable to find bound object"
		EndIf				
	EndFunction

	Function boundGetMaterial:Int(binding:Int,i:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getMaterial(i).handle
		Else
			Throw "Unable to find bound object"
		EndIf					
	EndFunction

	Function boundGetID:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getID()
		Else
			Throw "Unable to find bound object"
		EndIf				
	EndFunction

	Function boundGetBoundingBox:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getBoundingBox().handle
		Else
			Throw "Unable to find bound object"
		EndIf				
	EndFunction

	Function boundGetAutomaticCulling:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getAutomaticCulling()
		Else
			Throw "Unable to find bound object"
		EndIf				
	EndFunction
	
	Function boundGetAbsoluteTransformation:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getAbsoluteTransformation().handle
		Else
			Throw "Unable to find bound object"
		EndIf				
	EndFunction

	Function boundGetAbsolutePosition:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.getAbsolutePosition().handle
		Else
			Throw "Unable to find bound object"
		EndIf				
	EndFunction

	Function boundAddChild(binding:Int,child:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.addChild(ISceneNode.createFromHandle(child,False))
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundAddAnimator(binding:Int,animator:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.addAnimator(ISceneNodeAnimator.createFromHandle(animator,False))
		Else
			Throw "Unable to find bound object"
		EndIf	
	EndFunction

	Function boundIsVisible:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		If tempobj<>Null
	  		Return tempobj.isVisible()
		Else
			Throw "Unable to find bound object"
		EndIf
	EndFunction

	Function boundOnRegisterSceneNode(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.OnRegisterSceneNode()
		Else
			Throw "Unable to find bound object"
		EndIf
	EndFunction
	
	Function boundOnAnimate(binding:Int,timeMs:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.OnAnimate(timeMs)
		Else
			Throw "Unable to find bound object"
		EndIf
	EndFunction
		
	Function boundRender(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
  			' fire the render
	  		tempobj.render()
		Else
			Throw "Unable to find bound object"
		EndIf
	EndFunction

	Function boundIsDebugObject:Int(binding:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		Return tempobj.isDebugObject()
		Else
			Throw "Unable to find bound object"
		EndIf
	EndFunction

	Function boundSetIsDebugObject(binding:Int,bDebugObject:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.setIsDebugObject(bDebugObject)
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundSerializeAttributes(binding:Int,pIAttributes:Int,pSAttributeReadWriteOptions:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.serializeAttributes( ..
				IAttributes.createFromHandle(pIAttributes,False), ..
				SAttributeReadWriteOptions.createFromHandle(pSAttributeReadWriteOptions,False))
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundDeserializeAttributes(binding:Int,pIAttributes:Int,pSAttributeReadWriteOptions:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
	  		tempobj.deserializeAttributes( ..
				IAttributes.createFromHandle(pIAttributes,False), ..
				SAttributeReadWriteOptions.createFromHandle(pSAttributeReadWriteOptions,False))
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function boundClone:Int(binding:Int,pISceneNode_newParent:Int,pISceneManager_newManager:Int)
		' get the object whose method is firing
		Local tempobj:ISceneNode=ISceneNode(translate(binding))
		
		If tempobj<>Null
			Return tempobj.clone( ..
				ISceneNode.createFromHandle(pISceneNode_newParent,False), ..
				ISceneManager.createFromHandle(pISceneManager_newManager,False)).handle
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Method getBinderHandle:Int()
		Return BMXISceneNode_getBinder(handle)
	EndMethod				
EndType

' ------------------------------------------------------------------------
' default implementation of ISceneNode
' ------------------------------------------------------------------------

Type ISceneNodeDefault Extends ISceneNode
	Method getBoundingBox:AABbox3df()
		Return AABbox3df.createFromHandle(IrrScene_ISceneNode_getBoundingBox(handle))
	EndMethod
		
	Method render()
		IrrScene_ISceneNode_render(handle)
	EndMethod	
EndType
