' ------------------------------------------------------------------------------------
' irr::scene::ISceneNodeAnimator wrappers
' ------------------------------------------------------------------------------------
Public

Type ISceneNodeAnimator Extends IAttributeExchangingObject

	' --------------------------------------------------------------------------------
	' override these in extended type to implement new functionality
	' --------------------------------------------------------------------------------
	
	Method animateNode(node:ISceneNode,timeMs:Int)
		If Not bIsBound Then IrrScene_ISceneNodeAnimator_animateNode(handle,node.handle,timeMs)
	EndMethod

	Method getType:Int() ' returns ESCENE_NODE_ANIMATOR_TYPE
		If Not bIsBound Then
			Return IrrScene_ISceneNodeAnimator_getType(handle)
		Else
			Return IrrScene_BMXISceneNodeAnimator_getType(handle)
		EndIf
	EndMethod

	Method createClone:ISceneNodeAnimator(node:ISceneNode,newManager:ISceneManager = Null)
		If Not bIsBound Then
			Return ISceneNodeAnimator.createFromHandle(IrrScene_ISceneNodeAnimator_createClone(handle,node.handle,_HANDLE(newManager)),True)
		Else
			Return ISceneNodeAnimator.createFromHandle(IrrScene_BMXISceneNodeAnimator_createClone(handle,node.handle,_HANDLE(newManager)),True)
		EndIf
	EndMethod

	Method isEventReceiverEnabled:Int()
		If Not bIsBound Then Return IrrScene_ISceneNodeAnimatorisEventReceiverEnabled(handle) Else Return False
	EndMethod
	
	Method OnEvent:Int(event:SEvent)
		If Not bIsBound Then Return IrrScene_ISceneNodeAnimatorOnEvent(handle, event.handle) Else Return False
	EndMethod

	' --------------------------------------------------------------------------------
	' default base methods
	' --------------------------------------------------------------------------------

	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrScene_ISceneNodeAnimator_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrScene_ISceneNodeAnimator_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrScene_ISceneNodeAnimator_grab(handle)
	EndMethod
	
	Method Delete()
		If (mustdelete) Then drop()
	EndMethod
	' --------------------------------------------------------------------------------
	' creation routines
	' --------------------------------------------------------------------------------

	' this should only be called on an extended type
	Function create:ISceneNodeAnimator(pGenFunc:Byte Ptr) Final
		Return createFromHandle(IrrScene_BMXISceneNodeAnimator_new(),True,pGenFunc)
	EndFunction

	Function createFromHandle:ISceneNodeAnimator(pHandle:Int,bMustDel:Int,pGenFunc:Byte Ptr=Null)
		If Not pHandle Then Return Null
		Local oRetVal:ISceneNodeAnimator=Null
		
		If pGenFunc
			' use the funtion pointer to generate
			Local generate:ISceneNodeAnimator()=pGenFunc
		
			oRetVal=generate()
		Else
			oRetVal=New ISceneNodeAnimator
		EndIf

		' turn binding on if this is the first and there was a gen func passed in
		oRetVal.bIsBound=bMustDel And pGenFunc
		
		oRetVal.attach(pHandle,bMustDel)
		
		Return oRetVal
	EndFunction
	
	' --------------------------------------------------------------------------------
	' bindings
	' --------------------------------------------------------------------------------	

	Method bind(rec:Int) Final
		Super.bind(rec)
		IrrScene_BMXISceneNodeAnimator_bindAnimateNode( ..
			translate(rec).handle, ..
			OnAnimateNodeBinding)
		IrrScene_BMXISceneNodeAnimator_bindGetType( ..
			translate(rec).handle, ..
			OnGetTypeBinding)
		IrrScene_BMXISceneNodeAnimator_bindCreateClone( ..
			translate(rec).handle, ..
			OnCreateClone)
		IrrScene_BMXISceneNodeAnimator_bindIsEventReceiverEnabled( ..
			translate(rec).handle, ..
			OnIsEventReceiverEnabledBinding)
		IrrScene_BMXISceneNodeAnimator_bindOnEvent( ..
			translate(rec).handle, ..
			OnEventBinding)			
	EndMethod

	Function OnEventBinding:Int(binding:Int, pSEvent:Int)
		' get the object whose node is firing
		Local tempobj:ISceneNodeAnimator = ISceneNodeAnimator(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		Return tempobj.OnEvent(SEvent.createFromHandle(pSEvent, False))
		Else
			Throw "Unable to find bound object"
		EndIf
	EndFunction

	Function OnIsEventReceiverEnabledBinding:Int(binding:Int)
		' get the object whose node is firing
		Local tempobj:ISceneNodeAnimator = ISceneNodeAnimator(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		Return tempobj.isEventReceiverEnabled()
		Else
			Throw "Unable to find bound object"
		EndIf
	EndFunction

	Function OnGetTypeBinding:Int(binding:Int)
		' get the object whose node is firing
		Local tempobj:ISceneNodeAnimator=ISceneNodeAnimator(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		Return tempobj.getType()
		Else
			Throw "Unable to find bound object"
		EndIf
	EndFunction
	
	Function OnAnimateNodeBinding(binding:Int,node:Int,timeMs:Int) Final
		' get the object whose node is firing
		Local tempobj:ISceneNodeAnimator=ISceneNodeAnimator(translate(binding))
		
		If tempobj<>Null
	  		' get the node
  			Local tempnode:ISceneNode=ISceneNode.createFromHandle(node,False)
  			' fire the event
	  		tempobj.animateNode(tempnode,timeMs)
		Else
			Throw "Unable to find bound object"
		EndIf
	EndFunction
		
	Function OnCreateClone:Int(binding:Int,node:Int,newManager:Int)
		' get the object whose node is firing
		Local tempobj:ISceneNodeAnimator=ISceneNodeAnimator(translate(binding))
		If tempobj<>Null
			Return tempobj.createClone( ..
				ISceneNode.createFromHandle(node,False), ..
				ISceneManager.createFromHandle(newManager,False)).handle
		Else
			Throw "Unable to find bound object"
		EndIf
	EndFunction
	
	Method getBinderHandle:Int()
		Return IrrScene_BMXISceneNodeAnimator_getBinder(handle)
	EndMethod
		
EndType

