' ------------------------------------------------------------------------------------
' irr::video::IImage wrappers
'
' you cannot create IImages directly.  you must use IVideoDriver::CreateImage methods.
' when you are done with the image, the drop Function must be called.
' ------------------------------------------------------------------------------------
Public

Type IImage Extends IReferenceCounted Final

	Function createFromHandle:IImage(pHandle:Int,bMustDel:Int=True)
		If Not pHandle Then Return Null
		Local oRetVal:IImage=New IImage
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	' cast to either Short Ptr or Int Ptr if needed
	Method lock:Byte Ptr()
		Return IrrVideo_IImage_lock(handle)
	EndMethod

	Method unlock()
		IrrVideo_IImage_unlock(handle)
	EndMethod
	
	Method getDimension:Dimension2di()
		Return Dimension2di.createFromHandle(IrrVideo_IImage_getDimension(handle))
	EndMethod

	Method getBitsPerPixel:Int()
		Return IrrVideo_IImage_getBitsPerPixel(handle)
	EndMethod
	
	Method getBytesPerPixel:Int()
		Return IrrVideo_IImage_getBytesPerPixel(handle)
	EndMethod
	
	Method getImageDataSizeInBytes:Int()
		Return IrrVideo_IImage_getImageDataSizeInBytes(handle)
	EndMethod
	
	Method getImageDataSizeInPixels:Int()
		Return IrrVideo_IImage_getImageDataSizeInPixels(handle)
	EndMethod
	
	Method getPixel:SColor(x:Int,y:Int)
		Return SColor.createFromHandle(IrrVideo_IImage_getPixel(handle,x,y))
	EndMethod
	
	' returns ECOLOR_FORMAT enum
	Method getColorFormat:Int()
		Return IrrVideo_IImage_getColorFormat(handle)
	EndMethod

	Method getRedMask:Int()
		Return IrrVideo_IImage_getRedMask(handle)
	EndMethod
	
	Method getGreenMask:Int()
		Return IrrVideo_IImage_getGreenMask(handle)
	EndMethod

	Method getBlueMask:Int()
		Return IrrVideo_IImage_getBlueMask(handle)
	EndMethod

	Method getAlphaMask:Int()
		Return IrrVideo_IImage_getAlphaMask(handle)
	EndMethod

	Method getPitch:Int()
		Return IrrVideo_IImage_getPitch(handle)
	EndMethod

	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrVideo_IImage_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrVideo_IImage_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrVideo_IImage_grab(handle)
	EndMethod

	Method Delete()
		If (mustdelete) Then drop()
	EndMethod

	Method setPixel(x:Int,y:Int,color:SColor)
		IrrVideo_IImage_setPixel(handle,x,y,color.handle)
	EndMethod
	
	Method copyToScaling(target:Byte Ptr,width:Int,height:Int,ECOLOR_FORMAT:Int=ECF_A8R8G8B8,pitch:Int=0)
		IrrVideo_IImage_copyToScaling(handle,target,width,height,ECOLOR_FORMAT,pitch)
	EndMethod
	
	Method copyToScalingToImage(target:IImage)
		IrrVideo_IImage_copyToScalingToImage(handle,target.handle)
	EndMethod

	Method copyToImage(target:IImage, pos:Position2di = Null)
		IrrVideo_IImage_copyToImage(handle, _HANDLE(target), _HANDLE(pos))
	EndMethod
	
	Method copyToImageFromPart(target:IImage, pos:Position2di, sourceRect:Recti, clipRect:Recti = Null)
		IrrVideo_IImage_copyToImageFromPart(handle, _HANDLE(target), _HANDLE(pos), _HANDLE(sourceRect), _HANDLE(clipRect))
	EndMethod
	
	Method copyToWithAlpha(target:IImage, pos:Position2di, sourceRect:Recti, color:SColor, clipRect:Recti = Null)
		IrrVideo_IImage_copyToWithAlpha(handle, _HANDLE(target), _HANDLE(pos), _HANDLE(sourceRect), _HANDLE(color), _HANDLE(clipRect))
	EndMethod
	
	Method fill(color:SColor)
		IrrVideo_IImage_fill(handle, _HANDLE(color))
	EndMethod

EndType